@@ -7,56 +7,58 @@
 
 /* fn is "char *foo(int argc, const char *const *argv, const char *buff)" */
 #define OWLCMD_ARGS(name, fn, ctx, summary, usage, description) \
-        { name, summary, usage, description, ctx, \
+        { g_strdup(name), g_strdup(summary), g_strdup(usage), g_strdup(description), ctx, \
           NULL, fn, NULL, NULL, NULL, NULL, NULL, NULL }
 
 /* fn is "void foo(void)" */
 #define OWLCMD_VOID(name, fn, ctx, summary, usage, description) \
-        { name, summary, usage, description, ctx, \
+        { g_strdup(name), g_strdup(summary), g_strdup(usage), g_strdup(description), ctx, \
           NULL, NULL, fn, NULL, NULL, NULL, NULL, NULL }
 
 /* fn is "void foo(int)" */
 #define OWLCMD_INT(name, fn, ctx, summary, usage, description) \
-        { name, summary, usage, description, ctx, \
+        { g_strdup(name), g_strdup(summary), g_strdup(usage), g_strdup(description), ctx, \
           NULL, NULL, NULL, fn, NULL, NULL, NULL, NULL }
 
 #define OWLCMD_ALIAS(name, actualname) \
-        { name, OWL_CMD_ALIAS_SUMMARY_PREFIX actualname, "", "", OWL_CTX_ANY, \
-          actualname, NULL, NULL, NULL, NULL, NULL, NULL, NULL }
+        { g_strdup(name), g_strdup(OWL_CMD_ALIAS_SUMMARY_PREFIX actualname), g_strdup(""), g_strdup(""), OWL_CTX_ANY, \
+          g_strdup(actualname), NULL, NULL, NULL, NULL, NULL, NULL, NULL }
 
 /* fn is "char *foo(void *ctx, int argc, const char *const *argv, const char *buff)" */
 #define OWLCMD_ARGS_CTX(name, fn, ctx, summary, usage, description) \
-        { name, summary, usage, description, ctx, \
+        { g_strdup(name), g_strdup(summary), g_strdup(usage), g_strdup(description), ctx, \
           NULL, NULL, NULL, NULL, ((char*(*)(void*,int,const char*const *,const char*))fn), NULL, NULL, NULL }
 
 /* fn is "void foo(void)" */
 #define OWLCMD_VOID_CTX(name, fn, ctx, summary, usage, description) \
-        { name, summary, usage, description, ctx, \
+        { g_strdup(name), g_strdup(summary), g_strdup(usage), g_strdup(description), ctx, \
           NULL, NULL, NULL, NULL, NULL, ((void(*)(void*))(fn)), NULL, NULL }
 
 /* fn is "void foo(int)" */
 #define OWLCMD_INT_CTX(name, fn, ctx, summary, usage, description) \
-        { name, summary, usage, description, ctx, \
+        { g_strdup(name), g_strdup(summary), g_strdup(usage), g_strdup(description), ctx, \
           NULL, NULL, NULL, NULL, NULL, NULL, ((void(*)(void*,int))fn), NULL }
 
 
-const owl_cmd commands_to_init[]
-  = {
+void owl_cmd_add_defaults(owl_cmddict *cd)
+{
+  owl_cmd commands_to_init[] = {
+
   OWLCMD_ARGS("zlog", owl_command_zlog, OWL_CTX_ANY,
 	      "send a login or logout notification",
 	      "zlog in [tty]\nzlog out",
 	      "zlog in will send a login notification, zlog out will send a\n"
 	      "logout notification.  By default a login notification is sent\n"
-	      "when owl is started and a logout notification is sent when owl\n"
+	      "when BarnOwl is started and a logout notification is sent when owl\n"
 	      "is exited.  This behavior can be changed with the 'startuplogin'\n"
 	      "and 'shutdownlogout' variables.  If a tty is specified for zlog in\n"
-	      "then the owl variable 'tty' will be set to that string, causing\n"
+	      "then the BarnOwl variable 'tty' will be set to that string, causing\n"
 	      "it to be used as the zephyr location tty.\n"),
 
   OWLCMD_VOID("quit", owl_command_quit, OWL_CTX_ANY,
-	      "exit owl",
+	      "exit BarnOwl",
 	      "",
-	      "Exit owl and run any shutdown activities."),
+	      "Exit BarnOwl and run any shutdown activities."),
   OWLCMD_ALIAS("exit", "quit"),
   OWLCMD_ALIAS("q",    "quit"),
 
@@ -97,7 +99,7 @@ const owl_cmd commands_to_init[]
 
   OWLCMD_ARGS("unbindkey", owl_command_unbindkey, OWL_CTX_ANY,
 	      "removes a binding in a keymap",
-	      "bindkey <keymap> <keyseq>",
+	      "unbindkey <keymap> <keyseq>",
 	      "Removes a binding of a key sequence within a keymap.\n"
 	      "Use 'show keymaps' to see the existing keymaps.\n"
 	      "Ex.: unbindkey recv H\n\n"
@@ -177,20 +179,20 @@ const owl_cmd commands_to_init[]
 	      "are used print the value of all variables.\n"),
 
   OWLCMD_ARGS("startup", owl_command_startup, OWL_CTX_ANY,
-	      "run a command and set it to be run at every Owl startup",
+	      "run a command and set it to be run at every BarnOwl startup",
 	      "startup <commands> ...",
 	      "Everything on the command line after the startup command\n"
-	      "is executed as a normal owl command and is also placed in\n"
-	      "a file so that the command is executed every time owl\n"
+	      "is executed as a normal BarnOwl command and is also placed in\n"
+	      "a file so that the command is executed every time BarnOwl\n"
 	      "is started"),
 
   OWLCMD_ARGS("unstartup", owl_command_unstartup, OWL_CTX_ANY,
-	      "remove a command from the list of those to be run at Owl startup",
+	      "remove a command from the list of those to be run at BarnOwl startup",
 	      "unstartup <commands> ...",
 	      ""),
 
   OWLCMD_VOID("version", owl_command_version, OWL_CTX_ANY,
-	      "print the version of the running owl", "", ""),
+	      "print the version of the running BarnOwl", "", ""),
 
   OWLCMD_ARGS("subscribe", owl_command_subscribe, OWL_CTX_ANY,
 	      "subscribe to a zephyr class, instance, recipient",
@@ -201,7 +203,7 @@ const owl_cmd commands_to_init[]
 	      "If the -t option is present the subscription will\n"
 	      "only be temporary, i.e., it will not be written to\n"
 	      "the subscription file and will therefore not be\n"
-	      "present the next time owl is started.\n"),
+	      "present the next time BarnOwl is started.\n"),
   OWLCMD_ALIAS("sub", "subscribe"),
 
   OWLCMD_ARGS("unsubscribe", owl_command_unsubscribe, OWL_CTX_ANY,
@@ -213,7 +215,7 @@ const owl_cmd commands_to_init[]
 	      "If the -t option is present the unsubscription will\n"
 	      "only be temporary, i.e., it will not be updated in\n"
 	      "the subscription file and will therefore not be\n"
-	      "in effect the next time owl is started.\n"),
+	      "in effect the next time BarnOwl is started.\n"),
   OWLCMD_ALIAS("unsub", "unsubscribe"),
 
   OWLCMD_VOID("unsuball", owl_command_unsuball, OWL_CTX_ANY,
@@ -231,9 +233,9 @@ const owl_cmd commands_to_init[]
 	      "Dump messages in current view to the named file."),
 
   OWLCMD_ARGS("source", owl_command_source, OWL_CTX_ANY,
-	      "execute owl commands from a file",
+	      "execute BarnOwl commands from a file",
 	      "source <filename>",
-	      "Execute the owl commands in <filename>.\n"),
+	      "Execute the BarnOwl commands in <filename>.\n"),
 
   OWLCMD_ARGS("aim", owl_command_aim, OWL_CTX_INTERACTIVE,
 	      "AIM specific commands",
@@ -280,20 +282,20 @@ const owl_cmd commands_to_init[]
 
   OWLCMD_ARGS("punt", owl_command_punt, OWL_CTX_ANY,
 	      "suppress an arbitrary filter",
-	      "punt <filter-text>",
-	      "punt <filter-text (multiple words)>\n"
+	      "punt <filter-name>\n"
+	      "punt <filter-text (multiple words)>",
 	      "The punt command will suppress messages to the specified\n"
 	      "filter\n\n"
 	      "SEE ALSO:  unpunt, zpunt, show zpunts\n"),
 
   OWLCMD_ARGS("unpunt", owl_command_unpunt, OWL_CTX_ANY,
 	      "remove an entry from the punt list",
-	      "zpunt <filter-text>\n"
-	      "zpunt <filter-text>\n"
-	      "zpunt <number>\n",
+	      "unpunt <number>\n"
+	      "unpunt <filter-name>\n"
+	      "unpunt <filter-text (multiple words)>",
 	      "The unpunt command will remove an entry from the puntlist.\n"
-	      "The first two forms correspond to the first two forms of the :punt\n"
-	      "command. The latter allows you to remove a specific entry from the\n"
+	      "The last two forms correspond to the two forms of the :punt\n"
+	      "command. The first allows you to remove a specific entry from\n"
 	      "the list (see :show zpunts)\n\n"
 	      "SEE ALSO:  punt, zpunt, zunpunt, show zpunts\n"),
 
@@ -302,7 +304,7 @@ const owl_cmd commands_to_init[]
 	      "", ""),
   
   OWLCMD_ARGS("help", owl_command_help, OWL_CTX_INTERACTIVE,
-	      "display help on using owl",
+	      "display help on using BarnOwl",
 	      "help [command]", ""),
 
   OWLCMD_ARGS("zlist", owl_command_zlist, OWL_CTX_INTERACTIVE,
@@ -415,7 +417,7 @@ const owl_cmd commands_to_init[]
 	      "redraw the entire window", "", ""),
 
   OWLCMD_VOID("suspend", owl_command_suspend, OWL_CTX_ANY,
-	      "suspend owl", "", ""),
+	      "suspend BarnOwl", "", ""),
 
   OWLCMD_ARGS("echo", owl_command_echo, OWL_CTX_ANY,
 	      "pops up a message in popup window",
@@ -506,10 +508,10 @@ const owl_cmd commands_to_init[]
 	      "The file should contain a list of usernames, one per line."),
 
   OWLCMD_VOID("about", owl_command_about, OWL_CTX_INTERACTIVE,
-	      "print information about owl", "", ""),
+	      "print information about BarnOwl", "", ""),
 
   OWLCMD_VOID("status", owl_command_status, OWL_CTX_ANY,
-	      "print status information about the running owl", "", ""),
+	      "print status information about the running BarnOwl", "", ""),
   
   OWLCMD_ARGS("zlocate", owl_command_zlocate, OWL_CTX_INTERACTIVE,
 	      "locate a user",
@@ -587,7 +589,7 @@ const owl_cmd commands_to_init[]
 	      "\n"
 	      "The other usages listed above are abbreviated forms that simply set\n"
 	      "the filter of the current view. The -d option allows you to write a\n"
-              "filter expression that will be dynamically created by owl and then\n"
+              "filter expression that will be dynamically created by BarnOwl and then\n"
               "applied as the view's filter\n"
 	      "SEE ALSO: filter, viewclass, viewuser\n"),
 
@@ -653,7 +655,6 @@ const owl_cmd commands_to_init[]
 	      "show styles\n"
 	      "show subscriptions / show subs\n"
 	      "show terminal\n"
-	      "show timers\n"
 	      "show variables\n"
 	      "show variable <variable>\n"
 	      "show version\n"
@@ -674,7 +675,7 @@ const owl_cmd commands_to_init[]
 	      "Show styles will list the names of all styles available\n"
 	      "for formatting messages.\n\n"
 	      "Show variables will list the names of all variables.\n\n"
-	      "Show errors will show a list of errors encountered by Owl.\n\n"
+	      "Show errors will show a list of errors encountered by BarnOwl.\n\n"
 	      "SEE ALSO: filter, view, alias, bindkey, help\n"),
   
   OWLCMD_ARGS("delete", owl_command_delete, OWL_CTX_INTERACTIVE,
@@ -692,6 +693,15 @@ const owl_cmd commands_to_init[]
 	      "current view.\n"),
   OWLCMD_ALIAS("del", "delete"),
 
+  OWLCMD_ARGS("delete-and-expunge", owl_command_delete_and_expunge, OWL_CTX_INTERACTIVE,
+              "delete a message",
+              "delete-and-expunge [-id msgid] [-q | --quiet]",
+              "If no message id is specified the current message is deleted.\n"
+              "Otherwise the message with the given message id is deleted.\n"
+              "If --quiet is specified, then there is no message displayed on\n"
+              "success.\n"),
+  OWLCMD_ALIAS("delx", "delete-and-expunge"),
+
   OWLCMD_ARGS("undelete", owl_command_undelete, OWL_CTX_INTERACTIVE,
 	      "unmark a message for deletion",
 	      "undelete [ -id msgid ] [ --no-move ]\n"
@@ -1036,7 +1046,13 @@ const owl_cmd commands_to_init[]
   /* This line MUST be last! */
   { NULL, NULL, NULL, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }
 
-};
+  };
+
+  owl_cmddict_add_from_list(cd, commands_to_init);
+  owl_cmd *cmd;
+  for (cmd = commands_to_init; cmd->name != NULL; cmd++)
+    owl_cmd_cleanup(cmd);
+}
 
 void owl_command_info(void)
 {
@@ -1214,7 +1230,7 @@ char *owl_command_startup(int argc, const char *const *argv, const char *buff)
 
   ptr = skiptokens(buff, 1);
 
-  owl_function_command(ptr);
+  owl_function_command_norv(ptr);
   owl_function_addstartup(ptr);
 
   return(NULL);
@@ -1246,7 +1262,7 @@ char *owl_command_dump(int argc, const char *const *argv, const char *buff)
   }
   filename=owl_util_makepath(argv[1]);
   owl_function_dump(filename);
-  owl_free(filename);
+  g_free(filename);
   return(NULL);
 }
 
@@ -1273,7 +1289,7 @@ char *owl_command_next(int argc, const char *const *argv, const char *buff)
       last_if_none=1;
       argc-=1; argv+=1; 
     } else if (argc>=2 && !strcmp(argv[1], "--filter")) {
-      filter = owl_strdup(argv[2]);
+      filter = g_strdup(argv[2]);
       argc-=2; argv+=2; 
     } else if (argc>=2 && !strcmp(argv[1], "--smart-filter")) {
       filter = owl_function_smartfilter(0, 0);
@@ -1287,7 +1303,7 @@ char *owl_command_next(int argc, const char *const *argv, const char *buff)
     }
   }
   owl_function_nextmsg_full(filter, skip_deleted, last_if_none);
-  if (filter) owl_free(filter);
+  g_free(filter);
   return(NULL);
 }
 
@@ -1303,7 +1319,7 @@ char *owl_command_prev(int argc, const char *const *argv, const char *buff)
       first_if_none=1;
       argc-=1; argv+=1; 
     } else if (argc>=2 && !strcmp(argv[1], "--filter")) {
-      filter = owl_strdup(argv[2]);
+      filter = g_strdup(argv[2]);
       argc-=2; argv+=2; 
     } else if (argc>=2 && !strcmp(argv[1], "--smart-filter")) {
       filter = owl_function_smartfilter(0, 0);
@@ -1317,7 +1333,7 @@ char *owl_command_prev(int argc, const char *const *argv, const char *buff)
     }
   }
   owl_function_prevmsg_full(filter, skip_deleted, first_if_none);
-  if (filter) owl_free(filter);
+  g_free(filter);
   return(NULL);
 }
 
@@ -1327,7 +1343,7 @@ char *owl_command_smartnarrow(int argc, const char *const *argv, const char *buf
 
   char opt;
   int instance = 0, related = 0, i;
-  const char **tmp_argv = owl_malloc(sizeof(char *) * argc);
+  const char **tmp_argv = g_new(const char *, argc);
 
   static const struct option options[] = {
     {"instance", 0, 0, 'i'},
@@ -1356,16 +1372,16 @@ char *owl_command_smartnarrow(int argc, const char *const *argv, const char *buf
 
   if (filtname) {
     owl_function_change_currentview_filter(filtname);
-    owl_free(filtname);
+    g_free(filtname);
   }
 
 done:
-  owl_free(tmp_argv);
+  g_free(tmp_argv);
 
   return NULL;
 }
 
-char *owl_command_smartfilter(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_smartfilter(int argc, const char *const *argv, const char *buff)
 {
   char *filtname = NULL;
 
@@ -1405,14 +1421,14 @@ void owl_command_redisplay(void)
   owl_function_full_redisplay();
 }
 
-char *owl_command_get_shift(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_get_shift(int argc, const char *const *argv, const char *buff)
 {
   if(argc != 1)
   {
     owl_function_makemsg("Wrong number of arguments for %s", argv[0]);
     return NULL;
   }
-  return owl_sprintf("%d", owl_global_get_rightshift(&g));
+  return g_strdup_printf("%d", owl_global_get_rightshift(&g));
 }
 
 void owl_command_set_shift(int shift)
@@ -1564,6 +1580,7 @@ char *owl_command_set(int argc, const char *const *argv, const char *buff)
   const char *var, *val;
   int  silent=0;
   int requirebool=0;
+  owl_variable *v;
 
   if (argc == 1) {
     owl_function_printallvars();
@@ -1586,12 +1603,21 @@ char *owl_command_set(int argc, const char *const *argv, const char *buff)
     owl_function_makemsg("Wrong number of arguments for set command");
     return NULL;
   }
-  owl_variable_set_fromstring(owl_global_get_vardict(&g), var, val, !silent, requirebool);
+
+  v = owl_variable_get_var(owl_global_get_vardict(&g), var);
+  if (v == NULL) {
+    if (!silent) owl_function_error("Unknown variable '%s'", var);
+  } else if (requirebool && owl_variable_get_type(v) != OWL_VARIABLE_BOOL) {
+    if (!silent) owl_function_error("Variable '%s' is not a boolean", var);
+  } else {
+    owl_variable_set_fromstring(v, val, !silent);
+  }
   return NULL;
 }
 
 char *owl_command_unset(int argc, const char *const *argv, const char *buff)
 {
+  owl_variable *v;
   const char *var, *val;
   int  silent=0;
 
@@ -1606,14 +1632,23 @@ char *owl_command_unset(int argc, const char *const *argv, const char *buff)
     owl_function_makemsg("Wrong number of arguments for unset command");
     return NULL;
   }
-  owl_variable_set_fromstring(owl_global_get_vardict(&g), var, val, !silent, 1);
+
+  v = owl_variable_get_var(owl_global_get_vardict(&g), var);
+  if (v == NULL) {
+    if (!silent) owl_function_error("Unknown variable '%s'", var);
+  } else if (owl_variable_get_type(v) != OWL_VARIABLE_BOOL) {
+    if (!silent) owl_function_error("Variable '%s' is not a boolean", var);
+  } else {
+    owl_variable_set_fromstring(v, val, !silent);
+  }
   return NULL;
 }
 
 char *owl_command_print(int argc, const char *const *argv, const char *buff)
 {
   const char *var;
-  char valbuff[1024];
+  char *value;
+  const owl_variable *v;
 
   if (argc==1) {
     owl_function_printallvars();
@@ -1625,9 +1660,14 @@ char *owl_command_print(int argc, const char *const *argv, const char *buff)
 
   var=argv[1];
     
-  if (0 == owl_variable_get_tostring(owl_global_get_vardict(&g), 
-				     var, valbuff, 1024)) {
-    owl_function_makemsg("%s = '%s'", var, valbuff);
+  v = owl_variable_get_var(owl_global_get_vardict(&g), var);
+  if (v) {
+    value = owl_variable_get_tostring(v);
+    if (value == NULL)
+      owl_function_makemsg("%s = <null>", var);
+    else
+      owl_function_makemsg("%s = '%s'", var, value);
+    g_free(value);
   } else {
     owl_function_makemsg("Unknown variable '%s'.", var);
   }
@@ -1635,46 +1675,46 @@ char *owl_command_print(int argc, const char *const *argv, const char *buff)
 }
 
 
-char *owl_command_exec(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_exec(int argc, const char *const *argv, const char *buff)
 {
   return owl_function_exec(argc, argv, buff, OWL_OUTPUT_RETURN);
 }
 
-char *owl_command_pexec(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_pexec(int argc, const char *const *argv, const char *buff)
 {
   return owl_function_exec(argc, argv, buff, OWL_OUTPUT_POPUP);
 }
 
-char *owl_command_aexec(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_aexec(int argc, const char *const *argv, const char *buff)
 {
   return owl_function_exec(argc, argv, buff, OWL_OUTPUT_ADMINMSG);
 }
 
-char *owl_command_perl(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_perl(int argc, const char *const *argv, const char *buff)
 {
   return owl_function_perl(argc, argv, buff, OWL_OUTPUT_RETURN);
 }
 
-char *owl_command_pperl(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_pperl(int argc, const char *const *argv, const char *buff)
 {
   return owl_function_perl(argc, argv, buff, OWL_OUTPUT_POPUP);
 }
 
-char *owl_command_aperl(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_aperl(int argc, const char *const *argv, const char *buff)
 {
   return owl_function_perl(argc, argv, buff, OWL_OUTPUT_ADMINMSG);
 }
 
-char *owl_command_multi(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_multi(int argc, const char *const *argv, const char *buff)
 {
   char *lastrv = NULL, *newbuff;
   char **commands;
-  int  ncommands, i;
+  int  i;
   if (argc < 2) {
     owl_function_makemsg("Invalid arguments to 'multi' command.");    
     return NULL;
   }
-  newbuff = owl_strdup(skiptokens(buff, 1));
+  newbuff = g_strdup(skiptokens(buff, 1));
   if (!strcmp(argv[0], "(")) {
     for (i=strlen(newbuff)-1; i>=0; i--) {
       if (newbuff[i] == ')') {
@@ -1682,20 +1722,18 @@ char *owl_command_multi(int argc, const char *const *argv, const char *buff)
 	break;
       } else if (newbuff[i] != ' ') {
 	owl_function_makemsg("Invalid arguments to 'multi' command.");    
-	owl_free(newbuff);
+	g_free(newbuff);
 	return NULL;
       }
     }
   }
-  commands = atokenize(newbuff, ";", &ncommands);
-  for (i=0; i<ncommands; i++) {
-    if (lastrv) {
-      owl_free(lastrv);
-    }
+  commands = g_strsplit_set(newbuff, ";", 0);
+  for (i = 0; commands[i] != NULL; i++) {
+    g_free(lastrv);
     lastrv = owl_function_command(commands[i]);
   }
-  owl_free(newbuff);
-  atokenize_delete(commands, ncommands);
+  g_free(newbuff);
+  g_strfreev(commands);
   return lastrv;
 }
 
@@ -1743,7 +1781,7 @@ char *owl_command_unbindkey(int argc, const char *const *argv, const char *buf)
   int ret;
 
   if (argc < 3) {
-    owl_function_makemsg("Usage: bindkey <keymap> <binding>");
+    owl_function_makemsg("Usage: unbindkey <keymap> <binding>");
     return NULL;
   }
   km = owl_keyhandler_get_keymap(owl_global_get_keyhandler(&g), argv[1]);
@@ -1973,81 +2011,51 @@ char *owl_command_zwrite(int argc, const char *const *argv, const char *buff)
 
 char *owl_command_aimwrite(int argc, const char *const *argv, const char *buff)
 {
-  char *newbuff, *recip;
+  char *message = NULL;
+  GString *recip = g_string_new("");
   const char *const *myargv;
-  int i, j, myargc;
-  owl_message *m;
+  int myargc;
   
   if (!owl_global_is_aimloggedin(&g)) {
-    owl_function_makemsg("You are not logged in to AIM.");
-    return(NULL);
+    owl_function_error("You are not logged in to AIM.");
+    goto err;
   }
 
-  if (argc < 2) {
-    owl_function_makemsg("Not enough arguments to the aimwrite command.");
-    return(NULL);
-  }
-
-  myargv=argv;
-  if (argc<0) {
-    owl_function_error("Unbalanced quotes in aimwrite");
-    return(NULL);
-  }
-  myargc=argc;
-  if (myargc && *(myargv[0])!='-') {
-    myargc--;
-    myargv++;
-  }
+  /* Skip argv[0]. */
+  myargv = argv+1;
+  myargc = argc-1;
   while (myargc) {
     if (!strcmp(myargv[0], "-m")) {
-      if (myargc<2) {
-	break;
+      if (myargc <= 1) {
+	owl_function_error("No message specified.");
+	goto err;
       }
-
       /* Once we have -m, gobble up everything else on the line */
       myargv++;
       myargc--;
-      newbuff=owl_strdup("");
-      while (myargc) {
-	newbuff=owl_realloc(newbuff, strlen(newbuff)+strlen(myargv[0])+5);
-	strcat(newbuff, myargv[0]);
-	strcat(newbuff, " ");
-	myargc--;
-	myargv++;
-      }
-      if (strlen(newbuff) >= 1)
-	newbuff[strlen(newbuff) - 1] = '\0'; /* remove last space */
-
-      recip=owl_strdup(argv[1]);
-      owl_aim_send_im(recip, newbuff);
-      m=owl_function_make_outgoing_aim(newbuff, recip);
-      if (m) { 
-          owl_global_messagequeue_addmsg(&g, m);
-      } else {
-          owl_function_error("Could not create outgoing AIM message");
-      }
-
-      owl_free(recip);
-      owl_free(newbuff);
-      return(NULL);
+      message = g_strjoinv(" ", (char**)myargv);
+      break;
     } else {
-      /* we don't care */
+      /* squish arguments together to make one screenname w/o spaces for now */
+      g_string_append(recip, myargv[0]);
       myargv++;
       myargc--;
     }
   }
 
-  /* squish arguments together to make one screenname w/o spaces for now */
-  newbuff=owl_malloc(strlen(buff)+5);
-  sprintf(newbuff, "%s ", argv[0]);
-  j=argc-1;
-  for (i=0; i<j; i++) {
-    strcat(newbuff, argv[i+1]);
+  if (recip->str[0] == '\0') {
+    owl_function_error("No recipient specified");
+    goto err;
   }
-    
-  owl_function_aimwrite_setup(newbuff);
-  owl_free(newbuff);
-  return(NULL);
+
+  if (message != NULL)
+    owl_function_aimwrite(recip->str, message, false);
+  else
+    owl_function_aimwrite_setup(recip->str);
+ err:
+  g_string_free(recip, true);
+  g_free(message);
+  return NULL;
 }
 
 char *owl_command_loopwrite(int argc, const char *const *argv, const char *buff)
@@ -2133,7 +2141,7 @@ char *owl_command_view(int argc, const char *const *argv, const char *buff)
       char *foo;
       foo=owl_function_create_negative_filter(owl_view_get_filtname(owl_global_get_current_view(&g)));
       owl_function_change_currentview_filter(foo);
-      owl_free(foo);
+      g_free(foo);
       return(NULL);
     }
   }
@@ -2149,15 +2157,17 @@ char *owl_command_view(int argc, const char *const *argv, const char *buff)
     const char **myargv;
     int i;
 
-    myargv=owl_malloc((argc*sizeof(const char *))+50);
+    /* Allocate one more than argc for the trailing NULL. */
+    myargv = g_new0(const char*, argc+1);
     myargv[0]="";
     myargv[1]="owl-dynamic";
     for (i=2; i<argc; i++) {
       myargv[i]=argv[i];
     }
-    owl_function_create_filter(argc, myargv);
-    owl_function_change_currentview_filter("owl-dynamic");
-    owl_free(myargv);
+    if (owl_function_create_filter(argc, myargv)) {
+      owl_function_change_currentview_filter("owl-dynamic");
+    }
+    g_free(myargv);
     return NULL;
   }
 
@@ -2254,8 +2264,6 @@ char *owl_command_show(int argc, const char *const *argv, const char *buff)
     owl_function_show_colors();
   } else if (!strcmp(argv[1], "styles")) {
     owl_function_show_styles();
-  } else if (!strcmp(argv[1], "timers")) {
-    owl_function_show_timers();
   } else if (!strcmp(argv[1], "subs") || !strcmp(argv[1], "subscriptions")) {
     owl_function_getsubs();
   } else if (!strcmp(argv[1], "terminal") || !strcmp(argv[1], "term")) {
@@ -2292,7 +2300,7 @@ char *owl_command_viewclass(int argc, const char *const *argv, const char *buff)
   filtname = owl_function_classinstfilt(argv[1], NULL, owl_global_is_narrow_related(&g));
   if (filtname) {
     owl_function_change_currentview_filter(filtname);
-    owl_free(filtname);
+    g_free(filtname);
   }
   return NULL;
 }
@@ -2307,10 +2315,10 @@ char *owl_command_viewuser(int argc, const char *const *argv, const char *buff)
   }
   longuser = long_zuser(argv[1]);
   filtname = owl_function_zuserfilt(longuser);
-  owl_free(longuser);
+  g_free(longuser);
   if (filtname) {
     owl_function_change_currentview_filter(filtname);
-    owl_free(filtname);
+    g_free(filtname);
   }
   return NULL;
 }
@@ -2355,6 +2363,33 @@ char *owl_command_delete(int argc, const char *const *argv, const char *buff)
   return NULL;
 }
 
+char *owl_command_delete_and_expunge(int argc, const char *const *argv, const char *buff)
+{
+  bool exclaim_success = true;
+
+  if (argc > 1 && (!strcmp(argv[1], "-q") || !strcmp(argv[1], "--quiet"))) {
+    exclaim_success = false;
+    argc--;
+    argv++;
+  } else if (!strcmp(argv[argc - 1], "-q") || !strcmp(argv[argc - 1], "--quiet")) {
+    exclaim_success = false;
+    argc--;
+  }
+
+  if (argc == 1) {
+    owl_function_delete_and_expunge_cur(exclaim_success);
+    return NULL;
+  }
+
+  if (argc == 3 && (!strcmp(argv[1], "-id") || !strcmp(argv[1], "--id"))) {
+    owl_function_delete_and_expunge_by_id(atoi(argv[2]), exclaim_success);
+    return NULL;
+  }
+
+  owl_function_makemsg("Unknown arguments to delete-and-expunge command");
+  return NULL;
+}
+
 char *owl_command_undelete(int argc, const char *const *argv, const char *buff)
 {
   int move_after = 1;
@@ -2411,7 +2446,7 @@ char *owl_command_colorclass(int argc, const char *const *argv, const char *buff
   filtname=owl_function_classinstfilt(argv[1], NULL, owl_global_is_narrow_related(&g));
   if (filtname) {
     (void) owl_function_color_filter(filtname, argv[2], (argc == 4 ? argv[3] : NULL));
-    owl_free(filtname);
+    g_free(filtname);
   }
   return NULL;
 }
@@ -2491,34 +2526,28 @@ char *owl_command_unpunt(int argc, const char *const *argv, const char *buff)
 
 void owl_command_punt_unpunt(int argc, const char *const * argv, const char *buff, int unpunt)
 {
-  owl_list * fl;
-  owl_filter * f;
-  char * text;
+  GPtrArray * fl;
   int i;
 
   fl = owl_global_get_puntlist(&g);
   if(argc == 1) {
     owl_function_show_zpunts();
-  }
-
-  if(argc == 2) {
+  } else if(argc == 2) {
     /* Handle :unpunt <number> */
-    if(unpunt && (i=atoi(argv[1])) !=0) {
+    if (unpunt && (i = atoi(argv[1])) > 0) {
       i--;      /* Accept 1-based indexing */
-      if(i < owl_list_get_size(fl)) {
-        f = owl_list_get_element(fl, i);
-        owl_list_remove_element(fl, i);
-        owl_filter_delete(f);
+      if (i < fl->len) {
+        owl_filter_delete(g_ptr_array_remove_index(fl, i));
         return;
       } else {
-        owl_function_error("No such filter number: %d", i+1);
+        owl_function_makemsg("No such filter number: %d.", i+1);
       }
     }
-    text = owl_sprintf("filter %s", argv[1]);
-    owl_function_punt(text, unpunt);
-    owl_free(text);
+    const char *filter[] = {"filter", argv[1]};
+    owl_function_punt(2, filter, unpunt);
   } else {
-    owl_function_punt(skiptokens(buff, 1), unpunt);
+    /* Pass in argv[1]..argv[argc-1]. */
+    owl_function_punt(argc - 1, argv + 1, unpunt);
   }
 }
 
@@ -2531,22 +2560,20 @@ char *owl_command_getview(int argc, const char *const *argv, const char *buff)
     return NULL;
   }
   filtname = owl_view_get_filtname(owl_global_get_current_view(&g));
-  if (filtname) return owl_strdup(filtname);
+  if (filtname) return g_strdup(filtname);
   return NULL;
 }
 
-char *owl_command_getvar(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_getvar(int argc, const char *const *argv, const char *buff)
 {
-  char tmpbuff[1024];
+  const owl_variable *v;
   if (argc != 2) {
     owl_function_makemsg("Wrong number of arguments for %s", argv[0]);
     return NULL;
   }
-  if (owl_variable_get_tostring(owl_global_get_vardict(&g), 
-				argv[1], tmpbuff, 1024)) {
-    return NULL;
-  }
-  return owl_strdup(tmpbuff); 
+  v = owl_variable_get_var(owl_global_get_vardict(&g), argv[1]);
+  if (v == NULL) return NULL;
+  return owl_variable_get_tostring(v);
 }
 
 char *owl_command_getfilter(int argc, const char *const *argv, const char *buf)
@@ -2606,7 +2633,7 @@ char *owl_command_aimlogin(int argc, const char *const *argv, const char *buff)
   /* if we get two arguments, ask for the password */
   if (argc==2) {
     owl_editwin *e = owl_function_start_password("AIM Password: ");
-    owl_editwin_set_cbdata(e, owl_strdup(argv[1]), owl_free);
+    owl_editwin_set_cbdata(e, g_strdup(argv[1]), g_free);
     owl_editwin_set_callback(e, owl_callback_aimlogin);
     return(NULL);
   } else {
@@ -2626,7 +2653,7 @@ char *owl_command_aimlogout(int argc, const char *const *argv, const char *buff)
   return(NULL);
 }
 
-char *owl_command_getstyle(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_getstyle(int argc, const char *const *argv, const char *buff)
 {
   const char *stylename;
   if (argc != 1) {
@@ -2634,7 +2661,7 @@ char *owl_command_getstyle(int argc, const char *const *argv, const char *buff)
     return NULL;
   }
   stylename = owl_view_get_style_name(owl_global_get_current_view(&g));
-  if (stylename) return owl_strdup(stylename);
+  if (stylename) return g_strdup(stylename);
   return NULL;
 }
 
@@ -2664,13 +2691,12 @@ char *owl_command_add_cmd_history(int argc, const char *const *argv, const char
 
   ptr = skiptokens(buff, 1);
   hist = owl_global_get_cmd_history(&g);
-  owl_history_store(hist, ptr);
-  owl_history_reset(hist);
+  owl_history_store(hist, ptr, false);
   /* owl_function_makemsg("History '%s' stored successfully", ptr+1); */
   return NULL;
 }
 
-char *owl_command_with_history(int argc, const char *const *argv, const char *buff)
+CALLER_OWN char *owl_command_with_history(int argc, const char *const *argv, const char *buff)
 {
   owl_history *hist;
   const char *ptr;
@@ -2687,8 +2713,7 @@ char *owl_command_with_history(int argc, const char *const *argv, const char *bu
   }
 
   hist = owl_global_get_cmd_history(&g);
-  owl_history_store(hist, ptr);
-  owl_history_reset(hist);
+  owl_history_store(hist, ptr, false);
   return owl_function_command(ptr);
 }
 
@@ -2771,10 +2796,8 @@ void owl_command_edit_cancel(owl_editwin *e)
   owl_function_makemsg("Command cancelled.");
 
   hist = owl_editwin_get_history(e);
-  if (hist) {
-    owl_history_store(hist, owl_editwin_get_text(e));
-    owl_history_reset(hist);
-  }
+  if (hist)
+    owl_history_store(hist, owl_editwin_get_text(e), false);
 
   owl_global_pop_context(&g);
 }
@@ -2787,10 +2810,8 @@ void owl_command_edit_history_prev(owl_editwin *e)
   hist=owl_editwin_get_history(e);
   if (!hist)
     return;
-  if (!owl_history_is_touched(hist)) {
-    owl_history_store(hist, owl_editwin_get_text(e));
-    owl_history_set_partial(hist);
-  }
+  if (!owl_history_is_touched(hist))
+    owl_history_store(hist, owl_editwin_get_text(e), true);
   ptr=owl_history_get_prev(hist);
   if (ptr) {
     owl_editwin_clear(e);
@@ -2828,10 +2849,8 @@ void owl_command_edit_done(owl_editwin *e)
 {
   owl_history *hist=owl_editwin_get_history(e);
 
-  if (hist) {
-    owl_history_store(hist, owl_editwin_get_text(e));
-    owl_history_reset(hist);
-  }
+  if (hist)
+    owl_history_store(hist, owl_editwin_get_text(e), false);
 
   /* Take a reference to the editwin, so that it survives the pop
    * context. TODO: We should perhaps refcount or otherwise protect
