@@ -224,7 +224,7 @@ void owl_function_adminmsg(const char *header, const char *body)
  */
 void owl_function_add_outgoing_zephyrs(const owl_zwrite *z)
 {
-  if (z->cc || owl_zwrite_get_numrecips(z) == 0) {
+  if (z->cc && owl_zwrite_is_personal(z)) {
     /* create the message */
     owl_message *m = g_new(owl_message, 1);
     owl_message_create_from_zwrite(m, z, owl_zwrite_get_message(z), 0);
@@ -233,8 +233,13 @@ void owl_function_add_outgoing_zephyrs(const owl_zwrite *z)
   } else {
     int i;
     for (i = 0; i < owl_zwrite_get_numrecips(z); i++) {
+      owl_message *m;
+
+      if (!owl_zwrite_recip_is_personal(owl_zwrite_get_recip_n(z, i)))
+        continue;
+
       /* create the message */
-      owl_message *m = g_new(owl_message, 1);
+      m = g_new(owl_message, 1);
       owl_message_create_from_zwrite(m, z, owl_zwrite_get_message(z), i);
 
       owl_global_messagequeue_addmsg(&g, m);
@@ -380,11 +385,8 @@ void owl_function_zwrite(owl_zwrite *z, const char *msg)
   }
   owl_function_makemsg("Waiting for ack...");
 
-  /* If it's personal */
-  if (owl_zwrite_is_personal(z)) {
-    /* create the outgoing message */
-    owl_function_add_outgoing_zephyrs(z);
-  }
+  /* create the outgoing message */
+  owl_function_add_outgoing_zephyrs(z);
 }
 #else
 void owl_function_zwrite(owl_zwrite *z, const char *msg) {
@@ -410,13 +412,13 @@ void owl_function_zcrypt(owl_zwrite *z, const char *msg)
   old_msg = g_strdup(owl_zwrite_get_message(z));
 
   zcrypt = g_build_filename(owl_get_bindir(), "zcrypt", NULL);
-  argv[0] = "zcrypt";
+  argv[0] = zcrypt;
   argv[1] = "-E";
   argv[2] = "-c"; argv[3] = owl_zwrite_get_class(z);
   argv[4] = "-i"; argv[5] = owl_zwrite_get_instance(z);
   argv[6] = NULL;
 
-  rv = call_filter(zcrypt, argv, owl_zwrite_get_message(z), &cryptmsg, &status);
+  rv = call_filter(argv, owl_zwrite_get_message(z), &cryptmsg, &status);
 
   g_free(zcrypt);
 
@@ -434,15 +436,13 @@ void owl_function_zcrypt(owl_zwrite *z, const char *msg)
   owl_zwrite_send_message(z);
   owl_function_makemsg("Waiting for ack...");
 
-  /* If it's personal */
-  if (owl_zwrite_is_personal(z)) {
-    /* Create the outgoing message. Restore the un-crypted message for display. */
-    owl_zwrite_set_message_raw(z, old_msg);
-    owl_function_add_outgoing_zephyrs(z);
-  }
+  /* Create the outgoing message. Restore the un-crypted message for display. */
+  owl_zwrite_set_message_raw(z, old_msg);
+  owl_function_add_outgoing_zephyrs(z);
 
-  /* free the zwrite */
+  /* Clean up. */
   g_free(cryptmsg);
+  g_free(old_msg);
 }
 
 void owl_callback_aimwrite(owl_editwin *e) {
@@ -1431,6 +1431,7 @@ void owl_function_info(void)
     if (owl_message_is_direction_in(m)) {
       char *tmpbuff;
       int i, fields;
+      const char *f;
 
       n=owl_message_get_notice(m);
 
@@ -1474,8 +1475,9 @@ void owl_function_info(void)
 	fields=owl_zephyr_get_num_fields(n);
 	owl_fmtext_appendf_normal(&fm, "  Fields    : %i\n", fields);
 
-	for (i = 0; i < fields; i++) {
-	  tmpbuff = owl_zephyr_get_field_as_utf8(n, i + 1);
+	for (i = 0, f = owl_zephyr_first_raw_field(n); f != NULL;
+	     i++, f = owl_zephyr_next_raw_field(n, f)) {
+          tmpbuff = owl_zephyr_field_as_utf8(n, f);
 
 	  g_strdelimit(tmpbuff, "\n", '~');
 	  g_strdelimit(tmpbuff, "\r", '!');
@@ -2125,9 +2127,9 @@ void owl_function_change_currentview_filter(const char *filtname)
 }
 
 /* Create a new filter, or replace an existing one
- * with a new definition.
+ * with a new definition. Returns true on success.
  */
-void owl_function_create_filter(int argc, const char *const *argv)
+bool owl_function_create_filter(int argc, const char *const *argv)
 {
   owl_filter *f;
   const owl_view *v;
@@ -2135,7 +2137,7 @@ void owl_function_create_filter(int argc, const char *const *argv)
 
   if (argc < 2) {
     owl_function_error("Wrong number of arguments to filter command");
-    return;
+    return false;
   }
 
   owl_function_debugmsg("owl_function_create_filter: starting to create filter named %s", argv[1]);
@@ -2145,7 +2147,7 @@ void owl_function_create_filter(int argc, const char *const *argv)
   /* don't touch the all filter */
   if (!strcmp(argv[1], "all")) {
     owl_function_error("You may not change the 'all' filter.");
-    return;
+    return false;
   }
 
   /* deal with the case of trying change the filter color */
@@ -2153,36 +2155,36 @@ void owl_function_create_filter(int argc, const char *const *argv)
     f=owl_global_get_filter(&g, argv[1]);
     if (!f) {
       owl_function_error("The filter '%s' does not exist.", argv[1]);
-      return;
+      return false;
     }
     if (owl_util_string_to_color(argv[3])==OWL_COLOR_INVALID) {
       owl_function_error("The color '%s' is not available.", argv[3]);
-      return;
+      return false;
     }
     owl_filter_set_fgcolor(f, owl_util_string_to_color(argv[3]));
     owl_mainwin_redisplay(owl_global_get_mainwin(&g));
-    return;
+    return false;
   }
   if (argc==4 && !strcmp(argv[2], "-b")) {
     f=owl_global_get_filter(&g, argv[1]);
     if (!f) {
       owl_function_error("The filter '%s' does not exist.", argv[1]);
-      return;
+      return false;
     }
     if (owl_util_string_to_color(argv[3])==OWL_COLOR_INVALID) {
       owl_function_error("The color '%s' is not available.", argv[3]);
-      return;
+      return false;
     }
     owl_filter_set_bgcolor(f, owl_util_string_to_color(argv[3]));
     owl_mainwin_redisplay(owl_global_get_mainwin(&g));
-    return;
+    return true;
   }
 
   /* create the filter and check for errors */
   f = owl_filter_new(argv[1], argc-2, argv+2);
   if (f == NULL) {
     owl_function_error("Invalid filter");
-    return;
+    return false;
   }
 
   /* if the named filter is in use by the current view, remember it */
@@ -2203,6 +2205,7 @@ void owl_function_create_filter(int argc, const char *const *argv)
     owl_function_change_currentview_filter(argv[1]);
   }
   owl_mainwin_redisplay(owl_global_get_mainwin(&g));
+  return true;
 }
 
 /* If 'filtername' does not start with 'not-' create a filter named
