@@ -42,11 +42,11 @@ void owl_message_init(owl_message *m)
   m->delete=0;
 
   owl_message_set_hostname(m, "");
-  owl_list_create(&(m->attributes));
+  m->attributes = g_ptr_array_new();
   
   /* save the time */
   m->time=time(NULL);
-  m->timestr=owl_strdup(ctime(&(m->time)));
+  m->timestr=g_strdup(ctime(&(m->time)));
   m->timestr[strlen(m->timestr)-1]='\0';
 
   m->fmtext = NULL;
@@ -57,26 +57,25 @@ void owl_message_init(owl_message *m)
  */
 void owl_message_set_attribute(owl_message *m, const char *attrname, const char *attrvalue)
 {
-  int i, j;
+  int i;
   owl_pair *p = NULL, *pair = NULL;
 
   attrname = g_intern_string(attrname);
 
   /* look for an existing pair with this key, */
-  j=owl_list_get_size(&(m->attributes));
-  for (i=0; i<j; i++) {
-    p=owl_list_get_element(&(m->attributes), i);
+  for (i = 0; i < m->attributes->len; i++) {
+    p = m->attributes->pdata[i];
     if (owl_pair_get_key(p) == attrname) {
-      owl_free(owl_pair_get_value(p));
+      g_free(owl_pair_get_value(p));
       pair = p;
       break;
     }
   }
 
   if(pair ==  NULL) {
-    pair = owl_malloc(sizeof(owl_pair));
+    pair = g_new(owl_pair, 1);
     owl_pair_create(pair, attrname, NULL);
-    owl_list_append_element(&(m->attributes), pair);
+    g_ptr_array_add(m->attributes, pair);
   }
   owl_pair_set_value(pair, owl_validate_or_convert(attrvalue));
 }
@@ -86,7 +85,7 @@ void owl_message_set_attribute(owl_message *m, const char *attrname, const char
  */
 const char *owl_message_get_attribute_value(const owl_message *m, const char *attrname)
 {
-  int i, j;
+  int i;
   owl_pair *p;
   GQuark quark;
 
@@ -96,9 +95,8 @@ const char *owl_message_get_attribute_value(const owl_message *m, const char *at
     return NULL;
   attrname = g_quark_to_string(quark);
 
-  j=owl_list_get_size(&(m->attributes));
-  for (i=0; i<j; i++) {
-    p=owl_list_get_element(&(m->attributes), i);
+  for (i = 0; i < m->attributes->len; i++) {
+    p = m->attributes->pdata[i];
     if (owl_pair_get_key(p) == attrname) {
       return(owl_pair_get_value(p));
     }
@@ -117,23 +115,28 @@ const char *owl_message_get_attribute_value(const owl_message *m, const char *at
  * function to indent fmtext.
  */
 void owl_message_attributes_tofmtext(const owl_message *m, owl_fmtext *fm) {
-  int i, j;
+  int i;
   owl_pair *p;
-  char *buff;
+  char *buff, *tmpbuff;
 
   owl_fmtext_init_null(fm);
 
-  j=owl_list_get_size(&(m->attributes));
-  for (i=0; i<j; i++) {
-    p=owl_list_get_element(&(m->attributes), i);
-    buff=owl_sprintf("  %-15.15s: %-35.35s\n", owl_pair_get_key(p), owl_pair_get_value(p));
+  for (i = 0; i < m->attributes->len; i++) {
+    p = m->attributes->pdata[i];
+
+    tmpbuff = g_strdup(owl_pair_get_value(p));
+    g_strdelimit(tmpbuff, "\n", '~');
+    g_strdelimit(tmpbuff, "\r", '!');
+    buff = g_strdup_printf("  %-15.15s: %s\n", owl_pair_get_key(p), tmpbuff);
+    g_free(tmpbuff);
+
     if(buff == NULL) {
-      buff=owl_sprintf("  %-15.15s: %-35.35s\n", owl_pair_get_key(p), "<error>");
+      buff = g_strdup_printf("  %-15.15s: %s\n", owl_pair_get_key(p), "<error>");
       if(buff == NULL)
-        buff=owl_strdup("   <error>\n");
+        buff=g_strdup("   <error>\n");
     }
     owl_fmtext_append_normal(fm, buff);
-    owl_free(buff);
+    g_free(buff);
   }
 }
 
@@ -524,10 +527,8 @@ void owl_message_curs_waddstr(owl_message *m, WINDOW *win, int aline, int bline,
   
   owl_fmtext_truncate_lines(&(m->fmtext->fmtext), aline, bline-aline, &a);
   owl_fmtext_truncate_cols(&a, acol, bcol, &b);
-  owl_fmtext_colorize(&b, fgcolor);
-  owl_fmtext_colorizebg(&b, bgcolor);
 
-  owl_fmtext_curs_waddstr(&b, win);
+  owl_fmtext_curs_waddstr(&b, win, OWL_FMTEXT_ATTR_NONE, fgcolor, bgcolor);
 
   owl_fmtext_cleanup(&a);
   owl_fmtext_cleanup(&b);
@@ -575,7 +576,7 @@ int owl_message_is_mail(const owl_message *m)
 }
 
 /* caller must free return value. */
-char *owl_message_get_cc(const owl_message *m)
+CALLER_OWN char *owl_message_get_cc(const owl_message *m)
 {
   const char *cur;
   char *out, *end;
@@ -585,14 +586,14 @@ char *owl_message_get_cc(const owl_message *m)
   if (strncasecmp(cur, "cc:", 3)) return(NULL);
   cur+=3;
   while (*cur && *cur==' ') cur++;
-  out = owl_strdup(cur);
+  out = g_strdup(cur);
   end = strchr(out, '\n');
   if (end) end[0] = '\0';
   return(out);
 }
 
 /* caller must free return value */
-GList *owl_message_get_cc_without_recipient(const owl_message *m)
+CALLER_OWN GList *owl_message_get_cc_without_recipient(const owl_message *m)
 {
   char *cc, *shortuser, *recip;
   const char *user;
@@ -608,14 +609,14 @@ GList *owl_message_get_cc_without_recipient(const owl_message *m)
   while (user != NULL) {
     shortuser = short_zuser(user);
     if (strcasecmp(shortuser, recip) != 0) {
-      out = g_list_prepend(out, owl_strdup(user));
+      out = g_list_prepend(out, g_strdup(user));
     }
-    owl_free(shortuser);
+    g_free(shortuser);
     user = strtok(NULL, " ");
   }
 
-  owl_free(recip);
-  owl_free(cc);
+  g_free(recip);
+  g_free(cc);
 
   return(out);
 }
@@ -748,15 +749,15 @@ void owl_message_save_ccs(owl_message *m) {
     while(cc != NULL) {
       /* Collapse any identical entries */
       while (cc->next && strcasecmp(cc->data, cc->next->data) == 0) {
-        owl_free(cc->data);
+        g_free(cc->data);
         cc = g_list_delete_link(cc, cc);
       }
 
       tmp = short_zuser(cc->data);
       g_string_append(recips, tmp);
 
-      owl_free(tmp);
-      owl_free(cc->data);
+      g_free(tmp);
+      g_free(cc->data);
       cc = g_list_delete_link(cc, cc);
 
       if (cc)
@@ -776,7 +777,6 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
 #else /* !ZNOTICE_SOCKADDR */
   struct hostent *hent;
 #endif /* ZNOTICE_SOCKADDR */
-  const char *ptr;
   char *tmp, *tmp2;
   int len;
 
@@ -792,9 +792,9 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
   owl_zephyr_hackaway_cr(&(m->notice));
   
   /* save the time, we need to nuke the string saved by message_init */
-  if (m->timestr) owl_free(m->timestr);
+  if (m->timestr) g_free(m->timestr);
   m->time=n->z_time.tv_sec;
-  m->timestr=owl_strdup(ctime(&(m->time)));
+  m->timestr=g_strdup(ctime(&(m->time)));
   m->timestr[strlen(m->timestr)-1]='\0';
 
   /* set other info */
@@ -809,22 +809,18 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
   }
   owl_message_set_zsig(m, owl_zephyr_get_zsig(n, &len));
 
-  if ((ptr=strchr(n->z_recipient, '@'))!=NULL) {
-    owl_message_set_realm(m, ptr+1);
-  } else {
-    owl_message_set_realm(m, owl_zephyr_get_realm());
-  }
+  owl_message_set_realm(m, zuser_realm(n->z_recipient));
 
   /* Set the "isloginout" attribute if it's a login message */
   if (!strcasecmp(n->z_class, "login") || !strcasecmp(n->z_class, OWL_WEBZEPHYR_CLASS)) {
     if (!strcasecmp(n->z_opcode, "user_login") || !strcasecmp(n->z_opcode, "user_logout")) {
       tmp=owl_zephyr_get_field(n, 1);
       owl_message_set_attribute(m, "loginhost", tmp);
-      owl_free(tmp);
+      g_free(tmp);
 
       tmp=owl_zephyr_get_field(n, 3);
       owl_message_set_attribute(m, "logintty", tmp);
-      owl_free(tmp);
+      g_free(tmp);
     }
 
     if (!strcasecmp(n->z_opcode, "user_login")) {
@@ -835,10 +831,8 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
   }
 
   
-  /* set the "isprivate" attribute if it's a private zephyr.
-   ``private'' means recipient is non-empty and doesn't start wit 
-   `@' */
-  if (*n->z_recipient && *n->z_recipient != '@') {
+  /* set the "isprivate" attribute if it's a private zephyr. */
+  if (owl_zwrite_recip_is_personal(n->z_recipient)) {
     owl_message_set_isprivate(m);
   }
 
@@ -867,16 +861,16 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
   if (owl_global_is_newlinestrip(&g)) {
     tmp2=owl_util_stripnewlines(tmp);
     owl_message_set_body(m, tmp2);
-    owl_free(tmp2);
+    g_free(tmp2);
   } else {
     owl_message_set_body(m, tmp);
   }
-  owl_free(tmp);
+  g_free(tmp);
 
   /* if zcrypt is enabled try to decrypt the message */
   if (owl_global_is_zcrypt(&g) && !strcasecmp(n->z_opcode, "crypt")) {
     const char *argv[] = {
-      "zcrypt",
+      NULL,
       "-D",
       "-c", owl_message_get_class(m),
       "-i", owl_message_get_instance(m),
@@ -887,10 +881,11 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
     int status;
     char *zcrypt;
 
-    zcrypt = owl_sprintf("%s/zcrypt", owl_get_bindir());
+    zcrypt = g_build_filename(owl_get_bindir(), "zcrypt", NULL);
+    argv[0] = zcrypt;
 
-    rv = call_filter(zcrypt, argv, owl_message_get_body(m), &out, &status);
-    owl_free(zcrypt);
+    rv = call_filter(argv, owl_message_get_body(m), &out, &status);
+    g_free(zcrypt);
 
     if(!rv && !status) {
       int len = strlen(out);
@@ -898,9 +893,9 @@ void owl_message_create_from_znotice(owl_message *m, const ZNotice_t *n)
         out[len - 8] = 0;
       }
       owl_message_set_body(m, out);
-      owl_free(out);
+      g_free(out);
     } else if(out) {
-      owl_free(out);
+      g_free(out);
     }
   }
 
@@ -916,7 +911,6 @@ void owl_message_create_from_znotice(owl_message *m, const void *n)
 void owl_message_create_pseudo_zlogin(owl_message *m, int direction, const char *user, const char *host, const char *time, const char *tty)
 {
   char *longuser;
-  const char *ptr;
 
 #ifdef HAVE_LIBZEPHYR
   memset(&(m->notice), 0, sizeof(ZNotice_t));
@@ -945,24 +939,19 @@ void owl_message_create_pseudo_zlogin(owl_message *m, int direction, const char
     owl_message_set_islogout(m);
   }
 
-  if ((ptr=strchr(longuser, '@'))!=NULL) {
-    owl_message_set_realm(m, ptr+1);
-  } else {
-    owl_message_set_realm(m, owl_zephyr_get_realm());
-  }
+  owl_message_set_realm(m, zuser_realm(longuser));
 
   owl_message_set_body(m, "<uninitialized>");
 
   /* save the hostname */
   owl_function_debugmsg("create_pseudo_login: host is %s", host ? host : "");
   owl_message_set_hostname(m, host ? host : "");
-  owl_free(longuser);
+  g_free(longuser);
 }
 
-void owl_message_create_from_zwrite(owl_message *m, const owl_zwrite *z, const char *body)
+void owl_message_create_from_zwrite(owl_message *m, const owl_zwrite *z, const char *body, int recip_index)
 {
-  int ret;
-  char hostbuff[5000];
+  char *replyline;
   
   owl_message_init(m);
 
@@ -972,28 +961,33 @@ void owl_message_create_from_zwrite(owl_message *m, const owl_zwrite *z, const c
   owl_message_set_sender(m, owl_zephyr_get_sender());
   owl_message_set_class(m, owl_zwrite_get_class(z));
   owl_message_set_instance(m, owl_zwrite_get_instance(z));
-  if (owl_zwrite_get_numrecips(z)>0) {
-    char *longzuser = long_zuser(owl_zwrite_get_recip_n(z, 0));
-    owl_message_set_recipient(m,
-			      longzuser); /* only gets the first user, must fix */
-    owl_free(longzuser);
+  if (recip_index < owl_zwrite_get_numrecips(z)) {
+    char *zuser = owl_zwrite_get_recip_n_with_realm(z, recip_index);
+    char *longzuser = long_zuser(zuser);
+    owl_message_set_recipient(m, longzuser);
+    owl_message_set_realm(m, zuser_realm(longzuser));
+    g_free(longzuser);
+    g_free(zuser);
+  } else {
+    /* TODO: We should maybe munge this into the case above, as that comparison
+     * is a little overly clever. It's also not clear this codepath ever runs
+     * anyway. */
+    const char *realm = owl_zwrite_get_realm(z);
+    owl_message_set_realm(m, realm[0] ? realm : owl_zephyr_get_realm());
   }
   owl_message_set_opcode(m, owl_zwrite_get_opcode(z));
-  owl_message_set_realm(m, owl_zwrite_get_realm(z)); /* also a hack, but not here */
-  if(z->zwriteline) {
-    owl_message_set_zwriteline(m, z->zwriteline);
-  }
+
+  /* Although not strictly the zwriteline, anyone using the unsantized version
+   * of it probably has a bug. */
+  replyline = owl_zwrite_get_replyline(z, recip_index);
+  owl_message_set_zwriteline(m, replyline);
+  g_free(replyline);
+
   owl_message_set_body(m, body);
   owl_message_set_zsig(m, owl_zwrite_get_zsig(z));
   
   /* save the hostname */
-  ret=gethostname(hostbuff, MAXHOSTNAMELEN);
-  hostbuff[MAXHOSTNAMELEN]='\0';
-  if (ret) {
-    owl_message_set_hostname(m, "localhost");
-  } else {
-    owl_message_set_hostname(m, hostbuff);
-  }
+  owl_message_set_hostname(m, g_get_host_name());
 
   /* set the "isprivate" attribute if it's a private zephyr. */
   if (owl_zwrite_is_personal(z)) {
@@ -1005,24 +999,23 @@ void owl_message_create_from_zwrite(owl_message *m, const owl_zwrite *z, const c
 
 void owl_message_cleanup(owl_message *m)
 {
-  int i, j;
+  int i;
   owl_pair *p;
 #ifdef HAVE_LIBZEPHYR    
   if (owl_message_is_type_zephyr(m) && owl_message_is_direction_in(m)) {
     ZFreeNotice(&(m->notice));
   }
 #endif
-  if (m->timestr) owl_free(m->timestr);
+  if (m->timestr) g_free(m->timestr);
 
   /* free all the attributes */
-  j=owl_list_get_size(&(m->attributes));
-  for (i=0; i<j; i++) {
-    p=owl_list_get_element(&(m->attributes), i);
-    owl_free(owl_pair_get_value(p));
-    owl_free(p);
+  for (i = 0; i < m->attributes->len; i++) {
+    p = m->attributes->pdata[i];
+    g_free(owl_pair_get_value(p));
+    g_free(p);
   }
 
-  owl_list_cleanup(&(m->attributes), NULL);
+  g_ptr_array_free(m->attributes, true);
  
   owl_message_invalidate_format(m);
 }
@@ -1030,5 +1023,5 @@ void owl_message_cleanup(owl_message *m)
 void owl_message_delete(owl_message *m)
 {
   owl_message_cleanup(m);
-  owl_free(m);
+  g_free(m);
 }
