@@ -32,8 +32,6 @@
 int stderr_replace(void);
 #endif
 
-#define STDIN 0
-
 owl_global g;
 
 typedef struct _owl_options {
@@ -42,7 +40,6 @@ typedef struct _owl_options {
   char *tty;
   char *confdir;
   bool debug;
-  bool rm_debug;
 } owl_options;
 
 void usage(void)
@@ -51,7 +48,6 @@ void usage(void)
   fprintf(stderr, "Usage: barnowl [-n] [-d] [-D] [-v] [-h] [-c <configfile>] [-s <confdir>] [-t <ttyname>]\n");
   fprintf(stderr, "  -n,--no-subs        don't load zephyr subscriptions\n");
   fprintf(stderr, "  -d,--debug          enable debugging\n");
-  fprintf(stderr, "  -D,--remove-debug   enable debugging and delete previous debug file\n");
   fprintf(stderr, "  -v,--version        print the Barnowl version number and exit\n");
   fprintf(stderr, "  -h,--help           print this help message\n");
   fprintf(stderr, "  -c,--config-file    specify an alternate config file\n");
@@ -67,7 +63,6 @@ void owl_parse_options(int argc, char *argv[], owl_options *opts) {
     { "config-dir",      1, 0, 's' },
     { "tty",             1, 0, 't' },
     { "debug",           0, 0, 'd' },
-    { "remove-debug",    0, 0, 'D' },
     { "version",         0, 0, 'v' },
     { "help",            0, 0, 'h' },
     { NULL, 0, NULL, 0}
@@ -81,22 +76,19 @@ void owl_parse_options(int argc, char *argv[], owl_options *opts) {
       opts->load_initial_subs = 0;
       break;
     case 'c':
-      opts->configfile = owl_strdup(optarg);
+      opts->configfile = g_strdup(optarg);
       break;
     case 's':
-      opts->confdir = owl_strdup(optarg);
+      opts->confdir = g_strdup(optarg);
       break;
     case 't':
-      opts->tty = owl_strdup(optarg);
+      opts->tty = g_strdup(optarg);
       break;
-    case 'D':
-      opts->rm_debug = 1;
-      /* fallthrough */
     case 'd':
       opts->debug = 1;
       break;
     case 'v':
-      printf("This is barnowl version %s\n", OWL_VERSION_STRING);
+      printf("This is BarnOwl version %s\n", OWL_VERSION_STRING);
       exit(0);
     case 'h':
     default:
@@ -132,13 +124,15 @@ void owl_start_color(void) {
 void owl_start_curses(void) {
   struct termios tio;
   /* save initial terminal settings */
-  tcgetattr(0, owl_global_get_startup_tio(&g));
+  tcgetattr(STDIN_FILENO, owl_global_get_startup_tio(&g));
 
-  tcgetattr(0, &tio);
+  tcgetattr(STDIN_FILENO, &tio);
   tio.c_iflag &= ~(ISTRIP|IEXTEN);
-  tio.c_cc[VQUIT] = fpathconf(STDIN, _PC_VDISABLE);
-  tio.c_cc[VSUSP] = fpathconf(STDIN, _PC_VDISABLE);
-  tcsetattr(0, TCSAFLUSH, &tio);
+  tio.c_cc[VQUIT] = fpathconf(STDIN_FILENO, _PC_VDISABLE);
+  tio.c_cc[VSUSP] = fpathconf(STDIN_FILENO, _PC_VDISABLE);
+  tio.c_cc[VSTART] = fpathconf(STDIN_FILENO, _PC_VDISABLE);
+  tio.c_cc[VSTOP] = fpathconf(STDIN_FILENO, _PC_VDISABLE);
+  tcsetattr(STDIN_FILENO, TCSAFLUSH, &tio);
 
   /* screen init */
   initscr();
@@ -151,7 +145,7 @@ void owl_start_curses(void) {
 void owl_shutdown_curses(void) {
   endwin();
   /* restore terminal settings */
-  tcsetattr(0, TCSAFLUSH, owl_global_get_startup_tio(&g));
+  tcsetattr(STDIN_FILENO, TCSAFLUSH, owl_global_get_startup_tio(&g));
 }
 
 /*
@@ -165,7 +159,7 @@ void owl_shutdown_curses(void) {
  * Returns 1 if the message was added to the message list, and 0 if it
  * was ignored due to user settings or otherwise.
  */
-int owl_process_message(owl_message *m) {
+static int owl_process_message(owl_message *m) {
   const owl_filter *f;
   /* if this message it on the puntlist, nuke it and continue */
   if (owl_global_message_is_puntable(&g, m)) {
@@ -225,7 +219,7 @@ int owl_process_message(owl_message *m) {
     /* if it matches the alert filter, do the alert action */
     f=owl_global_get_filter(&g, owl_global_get_alert_filter(&g));
     if (f && owl_filter_message_match(f, m)) {
-      owl_function_command(owl_global_get_alert_action(&g));
+      owl_function_command_norv(owl_global_get_alert_action(&g));
     }
 
     /* if it's a zephyr login or logout, update the zbuddylist */
@@ -250,12 +244,19 @@ int owl_process_message(owl_message *m) {
   return 1;
 }
 
+static gboolean owl_process_messages_prepare(GSource *source, int *timeout) {
+  *timeout = -1;
+  return owl_global_messagequeue_pending(&g);
+}
+
+static gboolean owl_process_messages_check(GSource *source) {
+  return owl_global_messagequeue_pending(&g);
+}
+
 /*
  * Process any new messages we have waiting in the message queue.
- * Returns 1 if any messages were added to the message list, and 0 otherwise.
  */
-int owl_process_messages(owl_ps_action *d, void *p)
-{
+static gboolean owl_process_messages_dispatch(GSource *source, GSourceFunc callback, gpointer user_data) {
   int newmsgs=0;
   int followlast = owl_global_should_followlast(&g);
   owl_message *m;
@@ -279,16 +280,35 @@ int owl_process_messages(owl_ps_action *d, void *p)
     /* this should be optimized to not run if the new messages won't be displayed */
     owl_mainwin_redisplay(owl_global_get_mainwin(&g));
   }
-  return newmsgs;
+  return TRUE;
 }
 
-void owl_process_input(const owl_io_dispatch *d, void *data)
+static GSourceFuncs owl_process_messages_funcs = {
+  owl_process_messages_prepare,
+  owl_process_messages_check,
+  owl_process_messages_dispatch,
+  NULL
+};
+
+void owl_process_input_char(owl_input j)
 {
+  int ret;
+
+  owl_global_set_lastinputtime(&g, time(NULL));
+  ret = owl_keyhandler_process(owl_global_get_keyhandler(&g), j);
+  if (ret!=0 && ret!=1) {
+    owl_function_makemsg("Unable to handle keypress");
+  }
+}
+
+gboolean owl_process_input(GIOChannel *source, GIOCondition condition, void *data)
+{
+  owl_global *g = data;
   owl_input j;
 
   while (1) {
-    j.ch = wgetch(g.input_pad);
-    if (j.ch == ERR) return;
+    j.ch = wgetch(g->input_pad);
+    if (j.ch == ERR) return TRUE;
 
     j.uch = '\0';
     if (j.ch >= KEY_MIN && j.ch <= KEY_MAX) {
@@ -310,7 +330,7 @@ void owl_process_input(const owl_io_dispatch *d, void *data)
       else bytes = 1;
       
       for (i = 1; i < bytes; i++) {
-        int tmp = wgetch(g.input_pad);
+        int tmp = wgetch(g->input_pad);
         /* If what we got was not a byte, or not a continuation byte */
         if (tmp > 0xff || !(tmp & 0x80 && ~tmp & 0x40)) {
           /* ill-formed UTF-8 code unit subsequence, put back the
@@ -337,57 +357,74 @@ void owl_process_input(const owl_io_dispatch *d, void *data)
 
     owl_process_input_char(j);
   }
+  return TRUE;
 }
 
-void sig_handler(int sig, siginfo_t *si, void *data)
-{
-  if (sig==SIGWINCH) {
-    /* we can't inturrupt a malloc here, so it just sets a flag
-     * schedulding a resize for later
-     */
+static void sig_handler_main_thread(void *data) {
+  int sig = GPOINTER_TO_INT(data);
+
+  owl_function_debugmsg("Got signal %d", sig);
+  if (sig == SIGWINCH) {
     owl_function_resize();
-  } else if (sig==SIGPIPE || sig==SIGCHLD) {
-    /* Set a flag and some info that we got the sigpipe
-     * so we can record that we got it and why... */
-    owl_global_set_errsignal(&g, sig, si);
-  } else if (sig==SIGTERM || sig==SIGHUP) {
+  } else if (sig == SIGTERM || sig == SIGHUP) {
     owl_function_quit();
+  } else if (sig == SIGINT && owl_global_take_interrupt(&g)) {
+    owl_input in;
+    in.ch = in.uch = owl_global_get_startup_tio(&g)->c_cc[VINTR];
+    owl_process_input_char(in);
   }
 }
 
-void sigint_handler(int sig, siginfo_t *si, void *data)
-{
-  owl_global_set_interrupted(&g);
+static void sig_handler(const siginfo_t *siginfo, void *data) {
+  /* If it was an interrupt, set a flag so we can handle it earlier if
+   * needbe. sig_handler_main_thread will check the flag to make sure
+   * no one else took it. */
+  if (siginfo->si_signo == SIGINT) {
+    owl_global_add_interrupt(&g);
+  }
+  /* Send a message to the main thread. */
+  owl_select_post_task(sig_handler_main_thread,
+                       GINT_TO_POINTER(siginfo->si_signo), 
+		       NULL, g_main_context_default());
 }
 
-static int owl_errsignal_pre_select_action(owl_ps_action *a, void *data)
-{
-  siginfo_t si;
-  int signum;
-  if ((signum = owl_global_get_errsignal_and_clear(&g, &si)) > 0) {
-    owl_function_error("Got unexpected signal: %d %s  (code: %d band: %ld  errno: %d)",
-        signum, signum==SIGPIPE?"SIGPIPE":"SIG????",
-        si.si_code, si.si_band, si.si_errno);
-  }
-  return 0;
-}
+#define OR_DIE(s, syscall)       \
+  G_STMT_START {		 \
+    if ((syscall) == -1) {	 \
+      perror((s));		 \
+      exit(1);			 \
+    }				 \
+  } G_STMT_END
 
 void owl_register_signal_handlers(void) {
-  struct sigaction sigact;
+  struct sigaction sig_ignore = { .sa_handler = SIG_IGN };
+  struct sigaction sig_default = { .sa_handler = SIG_DFL };
+  sigset_t sigset;
+  int ret, i;
+  const int reset_signals[] = { SIGABRT, SIGBUS, SIGCHLD, SIGFPE, SIGILL,
+                                SIGQUIT, SIGSEGV, };
+  /* Don't bother resetting watched ones because owl_signal_init will. */
+  const int watch_signals[] = { SIGWINCH, SIGTERM, SIGHUP, SIGINT, };
 
-  /* signal handler */
-  /*sigact.sa_handler=sig_handler;*/
-  sigact.sa_sigaction=sig_handler;
-  sigemptyset(&sigact.sa_mask);
-  sigact.sa_flags=SA_SIGINFO;
-  sigaction(SIGWINCH, &sigact, NULL);
-  sigaction(SIGALRM, &sigact, NULL);
-  sigaction(SIGPIPE, &sigact, NULL);
-  sigaction(SIGTERM, &sigact, NULL);
-  sigaction(SIGHUP, &sigact, NULL);
+  /* Sanitize our signals; the mask and dispositions from our parent
+   * aren't really useful. Signal list taken from equivalent code in
+   * Chromium. */
+  OR_DIE("sigemptyset", sigemptyset(&sigset));
+  if ((ret = pthread_sigmask(SIG_SETMASK, &sigset, NULL)) != 0) {
+    errno = ret;
+    perror("pthread_sigmask");
+    exit(1);
+  }
+  for (i = 0; i < G_N_ELEMENTS(reset_signals); i++) {
+    OR_DIE("sigaction", sigaction(reset_signals[i], &sig_default, NULL));
+  }
 
-  sigact.sa_sigaction=sigint_handler;
-  sigaction(SIGINT, &sigact, NULL);
+  /* Turn off SIGPIPE; we check the return value of write. */
+  OR_DIE("sigaction", sigaction(SIGPIPE, &sig_ignore, NULL));
+
+  /* Register some signals with the signal thread. */
+  owl_signal_init(watch_signals, G_N_ELEMENTS(watch_signals),
+                  sig_handler, NULL);
 }
 
 #if OWL_STDERR_REDIR
@@ -412,72 +449,51 @@ int stderr_replace(void)
 }
 
 /* Sends stderr (read from rfd) messages to the error console */
-void stderr_redirect_handler(const owl_io_dispatch *d, void *data)
+gboolean stderr_redirect_handler(GIOChannel *source, GIOCondition condition, void *data)
 {
   int navail, bread;
   char buf[4096];
-  int rfd = d->fd;
+  int rfd = g_io_channel_unix_get_fd(source);
   char *err;
 
-  if (rfd<0) return;
+  /* TODO: Use g_io_channel_read_line? We'd have to be careful about
+   * blocking on the read. */
+
+  if (rfd<0) return TRUE;
   if (-1 == ioctl(rfd, FIONREAD, &navail)) {
-    return;
+    return TRUE;
   }
   /*owl_function_debugmsg("stderr_redirect: navail = %d\n", navail);*/
-  if (navail <= 0) return;
+  if (navail <= 0) return TRUE;
   if (navail > sizeof(buf)-1) {
     navail = sizeof(buf)-1;
   }
   bread = read(rfd, buf, navail);
-  if (buf[navail-1] != '\0') {
-    buf[navail] = '\0';
-  }
+  if (bread == -1)
+    return TRUE;
 
-  err = owl_sprintf("[stderr]\n%s", buf);
+  err = g_strdup_printf("[stderr]\n%.*s", bread, buf);
 
   owl_function_log_err(err);
-  owl_free(err);
+  g_free(err);
+  return TRUE;
 }
 
 #endif /* OWL_STDERR_REDIR */
 
-static int owl_refresh_pre_select_action(owl_ps_action *a, void *data)
-{
-  owl_colorpair_mgr *cpmgr;
-
-  /* if a resize has been scheduled, deal with it */
-  owl_global_check_resize(&g);
-  /* update the terminal if we need to */
-  owl_window_redraw_scheduled();
-  /* On colorpair shortage, reset and redraw /everything/. NOTE: if
-   * the current screen uses too many colorpairs, this draws
-   * everything twice. But this is unlikely; COLOR_PAIRS is 64 with
-   * 8+1 colors, and 256^2 with 256+1 colors. (+1 for default.) */
-  cpmgr = owl_global_get_colorpair_mgr(&g);
-  if (cpmgr->overflow) {
-    owl_function_debugmsg("colorpairs: color shortage; reset pairs and redraw. COLOR_PAIRS = %d", COLOR_PAIRS);
-    owl_fmtext_reset_colorpairs(cpmgr);
-    owl_function_full_redisplay();
-    owl_window_redraw_scheduled();
-  }
-  return 0;
-}
-
-
 int main(int argc, char **argv, char **env)
 {
-  int argcsave;
-  const char *const *argvsave;
+  int argc_copy;
+  char **argv_copy;
   char *perlout, *perlerr;
   const owl_style *s;
   const char *dir;
   owl_options opts;
+  GSource *source;
+  GIOChannel *channel;
 
-  if (!GLIB_CHECK_VERSION (2, 12, 0))
-    g_error ("GLib version 2.12.0 or above is needed.");
-
-  argcsave=argc;
-  argvsave=strs(argv);
+  argc_copy = argc;
+  argv_copy = g_strdupv(argv);
 
   setlocale(LC_ALL, "");
 
@@ -486,26 +502,31 @@ int main(int argc, char **argv, char **env)
   owl_parse_options(argc, argv, &opts);
   g.load_initial_subs = opts.load_initial_subs;
 
-  owl_register_signal_handlers();
   owl_start_curses();
 
   /* owl global init */
   owl_global_init(&g);
-  if (opts.rm_debug) unlink(OWL_DEBUG_FILE);
   if (opts.debug) owl_global_set_debug_on(&g);
   if (opts.confdir) owl_global_set_confdir(&g, opts.confdir);
   owl_function_debugmsg("startup: first available debugging message");
-  owl_global_set_startupargs(&g, argcsave, argvsave);
+  owl_global_set_startupargs(&g, argc_copy, argv_copy);
+  g_strfreev(argv_copy);
   owl_global_set_haveaim(&g);
 
+  owl_register_signal_handlers();
+
   /* register STDIN dispatch; throw away return, we won't need it */
-  owl_select_add_io_dispatch(STDIN, OWL_IO_READ, &owl_process_input, NULL, NULL);
+  channel = g_io_channel_unix_new(STDIN_FILENO);
+  g_io_add_watch(channel, G_IO_IN | G_IO_HUP | G_IO_ERR, &owl_process_input, &g);
+  g_io_channel_unref(channel);
   owl_zephyr_initialize();
 
 #if OWL_STDERR_REDIR
   /* Do this only after we've started curses up... */
   owl_function_debugmsg("startup: doing stderr redirection");
-  owl_select_add_io_dispatch(stderr_replace(), OWL_IO_READ, &stderr_redirect_handler, NULL, NULL);
+  channel = g_io_channel_unix_new(stderr_replace());
+  g_io_add_watch(channel, G_IO_IN | G_IO_HUP | G_IO_ERR, &stderr_redirect_handler, NULL);
+  g_io_channel_unref(channel);
 #endif
 
   /* create the owl directory, in case it does not exist */
@@ -520,7 +541,7 @@ int main(int argc, char **argv, char **env)
   } else {
     char *tty = owl_util_get_default_tty();
     owl_global_set_tty(&g, tty);
-    owl_free(tty);
+    g_free(tty);
   }
 
   /* Initialize perl */
@@ -556,13 +577,13 @@ int main(int argc, char **argv, char **env)
   /* execute the startup function in the configfile */
   owl_function_debugmsg("startup: executing perl startup, if applicable");
   perlout = owl_perlconfig_execute("BarnOwl::Hooks::_startup();");
-  if (perlout) owl_free(perlout);
+  g_free(perlout);
 
   /* welcome message */
   owl_function_debugmsg("startup: creating splash message");
   owl_function_adminmsg("",
     "-----------------------------------------------------------------------\n"
-    "Welcome to barnowl version " OWL_VERSION_STRING ".\n"
+    "Welcome to BarnOwl version " OWL_VERSION_STRING ".\n"
     "To see a quick introduction, type ':show quickstart'.                  \n"
     "Press 'h' for on-line help.                                            \n"
     "                                                                       \n"
@@ -589,15 +610,23 @@ int main(int argc, char **argv, char **env)
   owl_global_pop_context(&g);
   owl_global_push_context(&g, OWL_CTX_INTERACTIVE|OWL_CTX_RECV, NULL, "recv", NULL);
 
-  owl_select_add_pre_select_action(owl_refresh_pre_select_action, NULL, NULL);
-  owl_select_add_pre_select_action(owl_process_messages, NULL, NULL);
-  owl_select_add_pre_select_action(owl_errsignal_pre_select_action, NULL, NULL);
+  source = owl_window_redraw_source_new();
+  g_source_attach(source, NULL);
+  g_source_unref(source);
+
+  source = g_source_new(&owl_process_messages_funcs, sizeof(GSource));
+  g_source_attach(source, NULL);
+  g_source_unref(source);
+
+  owl_log_init();
 
   owl_function_debugmsg("startup: entering main loop");
   owl_select_run_loop();
 
   /* Shut down everything. */
   owl_zephyr_shutdown();
+  owl_signal_shutdown();
   owl_shutdown_curses();
+  owl_log_shutdown();
   return 0;
 }
