@@ -6,51 +6,53 @@
 #include "owl.h"
 
 #define OWLVAR_BOOL(name,default,summary,description) \
-        { name, OWL_VARIABLE_BOOL, NULL, default, "on,off", summary,description, NULL, \
+        { g_strdup(name), OWL_VARIABLE_BOOL, NULL, default, "on,off", g_strdup(summary), g_strdup(description), NULL, \
         NULL, NULL, NULL, NULL, NULL, NULL }
 
 #define OWLVAR_BOOL_FULL(name,default,summary,description,validate,set,get) \
-        { name, OWL_VARIABLE_BOOL, NULL, default, "on,off", summary,description, NULL, \
+        { g_strdup(name), OWL_VARIABLE_BOOL, NULL, default, "on,off", g_strdup(summary), g_strdup(description), NULL, \
         validate, set, NULL, get, NULL, NULL }
 
 #define OWLVAR_INT(name,default,summary,description) \
-        { name, OWL_VARIABLE_INT, NULL, default, "<int>", summary,description, NULL, \
+        { g_strdup(name), OWL_VARIABLE_INT, NULL, default, "<int>", g_strdup(summary), g_strdup(description), NULL, \
         NULL, NULL, NULL, NULL, NULL, NULL }
 
 #define OWLVAR_INT_FULL(name,default,summary,description,validset,validate,set,get) \
-        { name, OWL_VARIABLE_INT, NULL, default, validset, summary,description, NULL, \
+        { g_strdup(name), OWL_VARIABLE_INT, NULL, default, validset, g_strdup(summary), g_strdup(description), NULL, \
         validate, set, NULL, get, NULL, NULL }
 
 #define OWLVAR_PATH(name,default,summary,description) \
-        { name, OWL_VARIABLE_STRING, default, 0, "<path>", summary,description,  NULL, \
+        { g_strdup(name), OWL_VARIABLE_STRING, g_strdup(default), 0, "<path>", g_strdup(summary), g_strdup(description),  NULL, \
         NULL, NULL, NULL, NULL, NULL, NULL }
 
 #define OWLVAR_STRING(name,default,summary,description) \
-        { name, OWL_VARIABLE_STRING, default, 0, "<string>", summary,description, NULL, \
+        { g_strdup(name), OWL_VARIABLE_STRING, g_strdup(default), 0, "<string>", g_strdup(summary), g_strdup(description), NULL, \
         NULL, NULL, NULL, NULL, NULL, NULL }
 
-#define OWLVAR_STRING_FULL(name,default,summary,description,validate,set,get) \
-        { name, OWL_VARIABLE_STRING, default, 0, "<string>", summary,description, NULL, \
+#define OWLVAR_STRING_FULL(name,default,validset,summary,description,validate,set,get) \
+        { g_strdup(name), OWL_VARIABLE_STRING, g_strdup(default), 0, validset, g_strdup(summary), g_strdup(description), NULL, \
         validate, set, NULL, get, NULL, NULL }
 
 /* enums are really integers, but where validset is a comma-separated
  * list of strings which can be specified.  The tokens, starting at 0,
  * correspond to the values that may be specified. */
 #define OWLVAR_ENUM(name,default,summary,description,validset) \
-        { name, OWL_VARIABLE_INT, NULL, default, validset, summary,description, NULL, \
+        { g_strdup(name), OWL_VARIABLE_INT, NULL, default, validset, g_strdup(summary), g_strdup(description), NULL, \
         owl_variable_enum_validate, \
         NULL, owl_variable_enum_set_fromstring, \
         NULL, owl_variable_enum_get_tostring, \
         NULL }
 
 #define OWLVAR_ENUM_FULL(name,default,summary,description,validset,validate, set, get) \
-        { name, OWL_VARIABLE_INT, NULL, default, validset, summary,description, NULL, \
+        { g_strdup(name), OWL_VARIABLE_INT, NULL, default, validset, g_strdup(summary), g_strdup(description), NULL, \
         validate, \
         set, owl_variable_enum_set_fromstring, \
         get, owl_variable_enum_get_tostring, \
         NULL }
 
-static owl_variable variables_to_init[] = {
+int owl_variable_add_defaults(owl_vardict *vd)
+{
+  owl_variable variables_to_init[] = {
 
   OWLVAR_STRING( "personalbell" /* %OwlVarStub */, "off",
 		 "ring the terminal bell when personal messages are received",
@@ -67,10 +69,10 @@ static owl_variable variables_to_init[] = {
 		    NULL, owl_variable_debug_set, NULL),
 
   OWLVAR_BOOL( "startuplogin" /* %OwlVarStub */, 1,
-	       "send a login message when owl starts", "" ),
+	       "send a login message when BarnOwl starts", "" ),
 
   OWLVAR_BOOL( "shutdownlogout" /* %OwlVarStub */, 1,
-	       "send a logout message when owl exits", "" ),
+	       "send a logout message when BarnOwl exits", "" ),
 
   OWLVAR_BOOL( "rxping" /* %OwlVarStub */, 0,
 	       "display received pings", "" ),
@@ -134,7 +136,7 @@ static owl_variable variables_to_init[] = {
 
   OWLVAR_BOOL_FULL( "pseudologins" /* %OwlVarStub */, 0,
 		    "Enable zephyr pseudo logins",
-		    "When this is enabled, Owl will periodically check the zephyr\n"
+		    "When this is enabled, BarnOwl will periodically check the zephyr\n"
 		    "location of users in your .anyone file.  If a user is present\n"
 		    "but sent no login message, or a user is not present that sent no\n"
 		    "logout message, a pseudo login or logout message will be created\n",
@@ -142,8 +144,8 @@ static owl_variable variables_to_init[] = {
 
   OWLVAR_BOOL( "ignorelogins" /* %OwlVarStub */, 0,
 	       "Enable printing of login notifications",
-	       "When this is enabled, Owl will print login and logout notifications\n"
-	       "for AIM, zephyr, or other protocols.  If disabled Owl will not print\n"
+	       "When this is enabled, BarnOwl will print login and logout notifications\n"
+	       "for AIM, zephyr, or other protocols.  If disabled BarnOwl will not print\n"
 	       "login or logout notifications.\n"),
 
   OWLVAR_STRING( "logfilter" /* %OwlVarStub */, "",
@@ -157,14 +159,14 @@ static owl_variable variables_to_init[] = {
 
   OWLVAR_BOOL( "loglogins" /* %OwlVarStub */, 0,
 	       "Enable logging of login notifications",
-	       "When this is enabled, Owl will login login and logout notifications\n"
-	       "for AIM, zephyr, or other protocols.  If disabled Owl will not print\n"
+	       "When this is enabled, BarnOwl will log login and logout notifications\n"
+	       "for AIM, zephyr, or other protocols.  If disabled BarnOwl will not print\n"
 	       "login or logout notifications.\n"),
 
   OWLVAR_ENUM_FULL( "disable-ctrl-d" /* %OwlVarStub:lockout_ctrld */, 1,
 		    "don't send zephyrs on C-d",
-		    "If set to 'off', C-d won't send a zephyr from the edit\n"
-		    "window.  If set to 'on', C-d will always send a zephyr\n"
+		    "If set to 'on', C-d won't send a zephyr from the edit\n"
+		    "window.  If set to 'off', C-d will always send a zephyr\n"
 		    "being composed in the edit window.  If set to 'middle',\n"
 		    "C-d will only ever send a zephyr if the cursor is at\n"
 		    "the end of the message being composed.\n\n"
@@ -198,7 +200,7 @@ static owl_variable variables_to_init[] = {
 
   OWLVAR_PATH( "newmsgproc" /* %OwlVarStub:newmsgproc */, NULL,
 	       "name of a program to run when new messages are present",
-	       "The named program will be run when owl receives new\n"
+	       "The named program will be run when BarnOwl receives new\n"
 	       "messages.  It will not be run again until the first\n"
 	       "instance exits"),
 
@@ -226,7 +228,7 @@ static owl_variable variables_to_init[] = {
   OWLVAR_STRING( "appendtosepbar" /* %OwlVarStub */, "",
 	         "string to append to the end of the sepbar",
 		 "The sepbar is the bar separating the top and bottom\n"
-		 "of the owl screen.  Any string specified here will\n"
+		 "of the BarnOwl screen.  Any string specified here will\n"
 		 "be displayed on the right of the sepbar\n"),
 
   OWLVAR_BOOL( "zaway" /* %OwlVarStub */, 0,
@@ -262,10 +264,10 @@ static owl_variable variables_to_init[] = {
 		 "" ),
 
   OWLVAR_STRING( "alert_action" /* %OwlVarStub */, "nop",
-		 "owl command to execute for alert actions",
+		 "BarnOwl command to execute for alert actions",
 		 "" ),
 
-  OWLVAR_STRING_FULL( "tty" /* %OwlVarStub */, "", "tty name for zephyr location", "",
+  OWLVAR_STRING_FULL( "tty" /* %OwlVarStub */, "", "<string>", "tty name for zephyr location", "",
 		      NULL, owl_variable_tty_set, NULL),
 
   OWLVAR_STRING( "default_style" /* %OwlVarStub */, "default",
@@ -273,7 +275,7 @@ static owl_variable variables_to_init[] = {
 		 "This sets the default message formatting style.\n"
 		 "Styles may be created with the 'style' command.\n"
 		 "Some built-in styles include:\n"
-		 "   default  - the default owl formatting\n"
+		 "   default  - the default BarnOwl formatting\n"
 		 "   oneline  - one line per-message\n"
 		 "   perl     - legacy perl interface\n"
 		 "\nSEE ALSO: style, show styles, view -s <style>\n"
@@ -330,7 +332,7 @@ static owl_variable variables_to_init[] = {
 	       "This controls how the screen is scrolled as the\n"
 	       "cursor moves between messages being displayed.\n"
 	       "The following modes are supported:\n\n"
-	       "   normal      - This is the owl default.  Scrolling happens\n"
+	       "   normal      - This is the BarnOwl default.  Scrolling happens\n"
 	       "                 when it needs to, and an attempt is made to\n"
 	       "                 keep the current message roughly near\n"
 	       "                 the middle of the screen.\n"
@@ -369,11 +371,83 @@ static owl_variable variables_to_init[] = {
 	       "Note that this is currently risky as you might accidentally\n"
 	       "delete a message right as it came in.\n" ),
 
+  OWLVAR_STRING_FULL( "default_exposure" /* %OwlVarStub */, "",
+                      "none,opstaff,realm-visible,realm-announced,net-visible,net-announced",
+                      "controls the persistent value for exposure",
+                      "The default exposure level corresponds to the Zephyr exposure value\n"
+                      "in ~/.zephyr.vars.  Defaults to realm-visible if there is no value in\n"
+                      "~/.zephyr.vars.\n"
+                      "See the description of exposure for the values this can be.",
+                      NULL, owl_variable_default_exposure_set, owl_variable_default_exposure_get ),
+
+  OWLVAR_STRING_FULL( "exposure" /* %OwlVarStub */, "",
+                      "none,opstaff,realm-visible,realm-announced,net-visible,net-announced",
+                      "controls who can zlocate you",
+                      "The exposure level, defaulting to the value of default_exposure,\n"
+                      "can be one of the following (from least exposure to widest exposure,\n"
+                      "as listed in zctl(1)):\n"
+                      "\n"
+                      "   none            - This completely disables Zephyr for the user. \n"
+                      "                     The user is not registered with Zephyr.  No user\n"
+                      "                     location information is retained by Zephyr.  No\n"
+                      "                     login or logout announcements will be sent.  No\n"
+                      "                     subscriptions will be entered for the user, and\n"
+                      "                     no notices will be displayed by zwgc(1).\n"
+                      "   opstaff         - The user is registered with Zephyr.  No login or\n"
+                      "                     logout announcements will be sent, and location\n"
+                      "                     information will only be visible to Operations\n"
+                      "                     staff.  Default subscriptions and any additional\n"
+                      "                     personal subscriptions will be entered for the\n"
+                      "                     user.\n"
+                      "   realm-visible   - The user is registered with Zephyr.  User\n"
+                      "                     location information is retained by Zephyr and\n"
+                      "                     made available only to users within the user’s\n"
+                      "                     Kerberos realm.  No login or logout\n"
+                      "                     announcements will be sent.  This is the system\n"
+                      "                     default.  Default subscriptions and any\n"
+                      "                     additional personal subscriptions will be\n"
+                      "                     entered for the user.\n"
+                      "   realm-announced - The user is registered with Zephyr.  User\n"
+                      "                     location information is retained by Zephyr and\n"
+                      "                     made available only to users authenticated\n"
+                      "                     within the user’s Kerberos realm.  Login and\n"
+                      "                     logout announcements will be sent, but only to\n"
+                      "                     users within the user’s Kerberos realm who have\n"
+                      "                     explicitly requested such via subscriptions. \n"
+                      "                     Default subscriptions and any additional\n"
+                      "                     personal subscriptions will be entered for the\n"
+                      "                     user.\n"
+                      "   net-visible     - The user is registered with Zephyr.  User\n"
+                      "                     location information is retained by Zephyr and\n"
+                      "                     made available to any authenticated user who\n"
+                      "                     requests such.  Login and logout announcements\n"
+                      "                     will be sent only to users within the user’s\n"
+                      "                     Kerberos realm who have explicitly requested\n"
+                      "                     such via subscriptions.  Default subscriptions\n"
+                      "                     and any additional personal subscriptions will\n"
+                      "                     be entered for the user.\n"
+                      "   net-announced   - The user is registered with Zephyr.  User\n"
+                      "                     location information is retained by Zephyr and\n"
+                      "                     made available to any authenticated user who\n"
+                      "                     requests such.  Login and logout announcements\n"
+                      "                     will be sent to any user has requested such. \n"
+                      "                     Default subscriptions and any additional\n"
+                      "                     personal subscriptions will be entered for the\n"
+                      "                     user.\n",
+                      NULL, owl_variable_exposure_set, NULL /* use default for get */ ),
+
   /* This MUST be last... */
   { NULL, 0, NULL, 0, NULL, NULL, NULL, NULL,
     NULL, NULL, NULL, NULL, NULL, NULL }
 
-};
+  };
+
+  int ret = owl_variable_dict_add_from_list(vd, variables_to_init);
+  owl_variable *var;
+  for (var = variables_to_init; var->name != NULL; var++)
+    owl_variable_cleanup(var);
+  return ret;
+}
 
 /**************************************************************************/
 /*********************** SPECIFIC TO VARIABLES ****************************/
@@ -429,18 +503,17 @@ int owl_variable_aaway_set(owl_variable *v, const void *newval)
 
 int owl_variable_pseudologins_set(owl_variable *v, const void *newval)
 {
-  static owl_timer *timer = NULL;
+  static guint timer = 0;
   if (newval) {
     if (*(const int*)newval == 1) {
       owl_function_zephyr_buddy_check(0);
-      if (timer == NULL) {
-        timer = owl_select_add_timer("owl_zephyr_buddycheck_timer",
-                                     180, 180, owl_zephyr_buddycheck_timer, NULL, NULL);
+      if (timer == 0) {
+        timer = g_timeout_add_seconds(180, owl_zephyr_buddycheck_timer, NULL);
       }
     } else {
-      if (timer != NULL) {
-        owl_select_remove_timer(timer);
-        timer = NULL;
+      if (timer != 0) {
+        g_source_remove(timer);
+        timer = 0;
       }
     }
   }
@@ -465,25 +538,47 @@ int owl_variable_disable_ctrl_d_set(owl_variable *v, const void *newval)
 
 int owl_variable_tty_set(owl_variable *v, const void *newval)
 {
-  owl_zephyr_set_locationinfo(owl_global_get_hostname(&g), newval);
+  owl_zephyr_set_locationinfo(g_get_host_name(), newval);
   return(owl_variable_string_set_default(v, newval));
 }
 
+int owl_variable_default_exposure_set(owl_variable *v, const void *newval)
+{
+  return owl_zephyr_set_default_exposure(newval);
+}
+
+const void *owl_variable_default_exposure_get(const owl_variable *v)
+{
+  return owl_zephyr_get_default_exposure();
+}
+
+int owl_variable_exposure_set(owl_variable *v, const void *newval)
+{
+  int ret = owl_zephyr_set_exposure(newval);
+  if (ret != 0)
+    return ret;
+  return owl_variable_string_set_default(v, owl_zephyr_normalize_exposure(newval));
+}
 
 /**************************************************************************/
 /****************************** GENERAL ***********************************/
 /**************************************************************************/
 
 int owl_variable_dict_setup(owl_vardict *vd) {
+  owl_dict_create(vd);
+  return owl_variable_add_defaults(vd);
+}
+
+int owl_variable_dict_add_from_list(owl_vardict *vd, owl_variable *variables_to_init)
+{
   owl_variable *var, *cur;
-  if (owl_dict_create(vd)) return(-1);
   for (var = variables_to_init; var->name != NULL; var++) {
-    cur = owl_malloc(sizeof(owl_variable));
+    cur = g_new(owl_variable, 1);
     *cur = *var;
     /* strdup all the strings so we can delete them consistently. */
-    cur->name = owl_strdup(var->name);
-    cur->summary = owl_strdup(var->summary);
-    cur->description = owl_strdup(var->description);
+    cur->name = g_strdup(var->name);
+    cur->summary = g_strdup(var->summary);
+    cur->description = g_strdup(var->description);
     switch (cur->type) {
     case OWL_VARIABLE_OTHER:
       cur->set_fn(cur, cur->pval_default);
@@ -501,6 +596,7 @@ int owl_variable_dict_setup(owl_vardict *vd) {
 	cur->get_tostring_fn = owl_variable_string_get_tostring_default;      
       if (!cur->delete_fn)
 	cur->delete_fn = owl_variable_delete_default;
+      cur->pval_default = g_strdup(var->pval_default);
       cur->set_fn(cur, cur->pval_default);
       break;
     case OWL_VARIABLE_BOOL:
@@ -516,7 +612,7 @@ int owl_variable_dict_setup(owl_vardict *vd) {
 	cur->get_tostring_fn = owl_variable_bool_get_tostring_default;      
       if (!cur->delete_fn)
 	cur->delete_fn = owl_variable_delete_default;
-      cur->val = owl_malloc(sizeof(int));
+      cur->val = g_new(int, 1);
       cur->set_fn(cur, &cur->ival_default);
       break;
     case OWL_VARIABLE_INT:
@@ -532,7 +628,7 @@ int owl_variable_dict_setup(owl_vardict *vd) {
 	cur->get_tostring_fn = owl_variable_int_get_tostring_default;      
       if (!cur->delete_fn)
 	cur->delete_fn = owl_variable_delete_default;
-      cur->val = owl_malloc(sizeof(int));
+      cur->val = g_new(int, 1);
       cur->set_fn(cur, &cur->ival_default);
       break;
     default:
@@ -549,32 +645,34 @@ void owl_variable_dict_add_variable(owl_vardict * vardict,
   owl_dict_insert_element(vardict, var->name, var, (void (*)(void *))owl_variable_delete);
 }
 
-owl_variable * owl_variable_newvar(const char *name, const char *summary, const char * description) {
-  owl_variable * var = owl_malloc(sizeof(owl_variable));
-  memset(var, 0, sizeof(owl_variable));
-  var->name = owl_strdup(name);
-  var->summary = owl_strdup(summary);
-  var->description = owl_strdup(description);
+CALLER_OWN owl_variable *owl_variable_newvar(const char *name, const char *summary, const char *description)
+{
+  owl_variable * var = g_new0(owl_variable, 1);
+  var->name = g_strdup(name);
+  var->summary = g_strdup(summary);
+  var->description = g_strdup(description);
   return var;
 }
 
 void owl_variable_update(owl_variable *var, const char *summary, const char *desc) {
-  if(var->summary) owl_free(var->summary);
-  var->summary = owl_strdup(summary);
-  if(var->description) owl_free(var->description);
-  var->description = owl_strdup(desc);
+  g_free(var->summary);
+  var->summary = g_strdup(summary);
+  g_free(var->description);
+  var->description = g_strdup(desc);
 }
 
-void owl_variable_dict_newvar_string(owl_vardict * vd, const char *name, const char *summ, const char * desc, const char * initval) {
-  owl_variable *old = owl_variable_get_var(vd, name, OWL_VARIABLE_STRING);
-  if(old) {
+void owl_variable_dict_newvar_string(owl_vardict *vd, const char *name, const char *summ, const char *desc, const char *initval)
+{
+  owl_variable *old = owl_variable_get_var(vd, name);
+  if (old && owl_variable_get_type(old) == OWL_VARIABLE_STRING) {
     owl_variable_update(old, summ, desc);
-    if(old->pval_default) owl_free(old->pval_default);
-    old->pval_default = owl_strdup(initval);
+    g_free(old->pval_default);
+    old->pval_default = g_strdup(initval);
   } else {
     owl_variable * var = owl_variable_newvar(name, summ, desc);
     var->type = OWL_VARIABLE_STRING;
-    var->pval_default = owl_strdup(initval);
+    var->validsettings = "<string>";
+    var->pval_default = g_strdup(initval);
     var->set_fn = owl_variable_string_set_default;
     var->set_fromstring_fn = owl_variable_string_set_fromstring_default;
     var->get_fn = owl_variable_get_default;
@@ -585,14 +683,16 @@ void owl_variable_dict_newvar_string(owl_vardict * vd, const char *name, const c
   }
 }
 
-void owl_variable_dict_newvar_int(owl_vardict * vd, const char *name, const char *summ, const char * desc, int initval) {
-  owl_variable *old = owl_variable_get_var(vd, name, OWL_VARIABLE_INT);
-  if(old) {
+void owl_variable_dict_newvar_int(owl_vardict *vd, const char *name, const char *summ, const char *desc, int initval)
+{
+  owl_variable *old = owl_variable_get_var(vd, name);
+  if (old && owl_variable_get_type(old) == OWL_VARIABLE_INT) {
     owl_variable_update(old, summ, desc);
     old->ival_default = initval;
   } else {
     owl_variable * var = owl_variable_newvar(name, summ, desc);
     var->type = OWL_VARIABLE_INT;
+    var->validsettings = "<int>";
     var->ival_default = initval;
     var->validate_fn = owl_variable_int_validate_default;
     var->set_fn = owl_variable_int_set_default;
@@ -600,20 +700,22 @@ void owl_variable_dict_newvar_int(owl_vardict * vd, const char *name, const char
     var->get_fn = owl_variable_get_default;
     var->get_tostring_fn = owl_variable_int_get_tostring_default;
     var->delete_fn = owl_variable_delete_default;
-    var->val = owl_malloc(sizeof(int));
+    var->val = g_new(int, 1);
     var->set_fn(var, &initval);
     owl_variable_dict_add_variable(vd, var);
   }
 }
 
-void owl_variable_dict_newvar_bool(owl_vardict * vd, const char *name, const char *summ, const char * desc, int initval) {
-  owl_variable *old = owl_variable_get_var(vd, name, OWL_VARIABLE_BOOL);
-  if(old) {
+void owl_variable_dict_newvar_bool(owl_vardict *vd, const char *name, const char *summ, const char *desc, int initval)
+{
+  owl_variable *old = owl_variable_get_var(vd, name);
+  if (old && owl_variable_get_type(old) == OWL_VARIABLE_BOOL) {
     owl_variable_update(old, summ, desc);
     old->ival_default = initval;
   } else {
     owl_variable * var = owl_variable_newvar(name, summ, desc);
     var->type = OWL_VARIABLE_BOOL;
+    var->validsettings = "on,off";
     var->ival_default = initval;
     var->validate_fn = owl_variable_bool_validate_default;
     var->set_fn = owl_variable_bool_set_default;
@@ -621,7 +723,7 @@ void owl_variable_dict_newvar_bool(owl_vardict * vd, const char *name, const cha
     var->get_fn = owl_variable_get_default;
     var->get_tostring_fn = owl_variable_bool_get_tostring_default;
     var->delete_fn = owl_variable_delete_default;
-    var->val = owl_malloc(sizeof(int));
+    var->val = g_new(int, 1);
     var->set_fn(var, &initval);
     owl_variable_dict_add_variable(vd, var);
   }
@@ -632,26 +734,32 @@ void owl_variable_dict_cleanup(owl_vardict *d)
   owl_dict_cleanup(d, (void (*)(void *))owl_variable_delete);
 }
 
-/* free the list with owl_variable_dict_namelist_cleanup */
-void owl_variable_dict_get_names(const owl_vardict *d, owl_list *l) {
-  owl_dict_get_keys(d, l);
+CALLER_OWN GPtrArray *owl_variable_dict_get_names(const owl_vardict *d) {
+  return owl_dict_get_keys(d);
 }
 
-void owl_variable_dict_namelist_cleanup(owl_list *l)
+void owl_variable_cleanup(owl_variable *v)
 {
-  owl_list_cleanup(l, owl_free);
+  if (v->delete_fn) v->delete_fn(v);
+  g_free(v->name);
+  g_free(v->summary);
+  g_free(v->description);
+  if (v->type == OWL_VARIABLE_STRING)
+    g_free(v->pval_default);
 }
 
 void owl_variable_delete(owl_variable *v)
 {
-  if (v->delete_fn) v->delete_fn(v);
-  owl_free(v->name);
-  owl_free(v->summary);
-  owl_free(v->description);
-  owl_free(v);
+  owl_variable_cleanup(v);
+  g_free(v);
 }
 
 
+const char *owl_variable_get_name(const owl_variable *v)
+{
+  return v->name;
+}
+
 const char *owl_variable_get_description(const owl_variable *v) {
   return v->description;
 }
@@ -661,186 +769,165 @@ const char *owl_variable_get_summary(const owl_variable *v) {
 }
 
 const char *owl_variable_get_validsettings(const owl_variable *v) {
-  if (v->validsettings) {
-    return v->validsettings;
-  } else {
-    return "";
-  }
+  return v->validsettings;
+}
+
+int owl_variable_get_type(const owl_variable *v)
+{
+  return v->type;
 }
 
 /* functions for getting and setting variable values */
 
 /* returns 0 on success, prints a status msg if msg is true */
-int owl_variable_set_fromstring(owl_vardict *d, const char *name, const char *value, int msg, int requirebool) {
-  owl_variable *v;
-  char buff2[1024];
-  if (!name) return(-1);
-  v = owl_dict_find_element(d, name);
-  if (v == NULL) {
-    if (msg) owl_function_error("Unknown variable %s", name);
-    return -1;
-  }
+int owl_variable_set_fromstring(owl_variable *v, const char *value, int msg) {
+  char *tostring;
   if (!v->set_fromstring_fn) {
-    if (msg) owl_function_error("Variable %s is read-only", name);
-    return -1;   
-  }
-  if (requirebool && v->type!=OWL_VARIABLE_BOOL) {
-    if (msg) owl_function_error("Variable %s is not a boolean", name);
+    if (msg) owl_function_error("Variable %s is read-only", owl_variable_get_name(v));
     return -1;   
   }
   if (0 != v->set_fromstring_fn(v, value)) {
-    if (msg) owl_function_error("Unable to set %s (must be %s)", name, 
-				  owl_variable_get_validsettings(v));
+    if (msg) owl_function_error("Unable to set %s (must be %s)", owl_variable_get_name(v),
+                                owl_variable_get_validsettings(v));
     return -1;
   }
-  if (msg && v->get_tostring_fn) {
-    v->get_tostring_fn(v, buff2, 1024, v->val);
-    owl_function_makemsg("%s = '%s'", name, buff2);
+  if (msg) {
+    tostring = v->get_tostring_fn(v, v->get_fn(v));
+    if (tostring)
+      owl_function_makemsg("%s = '%s'", owl_variable_get_name(v), tostring);
+    else
+      owl_function_makemsg("%s = <null>", owl_variable_get_name(v));
+    g_free(tostring);
   }    
   return 0;
 }
  
-int owl_variable_set_string(owl_vardict *d, const char *name, const char *newval) {
-  owl_variable *v;
-  if (!name) return(-1);
-  v = owl_dict_find_element(d, name);
-  if (v == NULL || !v->set_fn) return(-1);
-  if (v->type!=OWL_VARIABLE_STRING) return(-1);
+int owl_variable_set_string(owl_variable *v, const char *newval)
+{
+  if (v->type != OWL_VARIABLE_STRING) return -1;
   return v->set_fn(v, newval);
 }
  
-int owl_variable_set_int(owl_vardict *d, const char *name, int newval) {
-  owl_variable *v;
-  if (!name) return(-1);
-  v = owl_dict_find_element(d, name);
-  if (v == NULL || !v->set_fn) return(-1);
-  if (v->type!=OWL_VARIABLE_INT && v->type!=OWL_VARIABLE_BOOL) return(-1);
+int owl_variable_set_int(owl_variable *v, int newval)
+{
+  if (v->type != OWL_VARIABLE_INT && v->type != OWL_VARIABLE_BOOL) return -1;
   return v->set_fn(v, &newval);
 }
  
-int owl_variable_set_bool_on(owl_vardict *d, const char *name) {
-  return owl_variable_set_int(d,name,1);
+int owl_variable_set_bool_on(owl_variable *v)
+{
+  if (v->type != OWL_VARIABLE_BOOL) return -1;
+  return owl_variable_set_int(v, true);
 }
 
-int owl_variable_set_bool_off(owl_vardict *d, const char *name) {
-  return owl_variable_set_int(d,name,0);
+int owl_variable_set_bool_off(owl_variable *v)
+{
+  if (v->type != OWL_VARIABLE_BOOL) return -1;
+  return owl_variable_set_int(v, false);
 }
 
-int owl_variable_get_tostring(const owl_vardict *d, const char *name, char *buf, int bufsize) {
-  owl_variable *v;
-  if (!name) return(-1);
-  v = owl_dict_find_element(d, name);
-  if (v == NULL || !v->get_tostring_fn) return(-1);
-  return v->get_tostring_fn(v, buf, bufsize, v->val);
+CALLER_OWN char *owl_variable_get_tostring(const owl_variable *v)
+{
+  return v->get_tostring_fn(v, v->get_fn(v));
 }
 
-int owl_variable_get_default_tostring(const owl_vardict *d, const char *name, char *buf, int bufsize) {
-  owl_variable *v;
-  if (!name) return(-1);
-  v = owl_dict_find_element(d, name);
-  if (v == NULL || !v->get_tostring_fn) return(-1);
+CALLER_OWN char *owl_variable_get_default_tostring(const owl_variable *v)
+{
   if (v->type == OWL_VARIABLE_INT || v->type == OWL_VARIABLE_BOOL) {
-    return v->get_tostring_fn(v, buf, bufsize, &(v->ival_default));
+    return v->get_tostring_fn(v, &(v->ival_default));
   } else {
-    return v->get_tostring_fn(v, buf, bufsize, v->pval_default);
+    return v->get_tostring_fn(v, v->pval_default);
   }
 }
 
-owl_variable *owl_variable_get_var(const owl_vardict *d, const char *name, int require_type) {
-  owl_variable *v;
-  if (!name) return(NULL);
-  v = owl_dict_find_element(d, name);
-  if (v == NULL || !v->get_fn || v->type != require_type) return(NULL);
-  return v;
+owl_variable *owl_variable_get_var(const owl_vardict *d, const char *name)
+{
+  return owl_dict_find_element(d, name);
 }
 
 /* returns a reference */
-const void *owl_variable_get(const owl_vardict *d, const char *name, int require_type) {
-  owl_variable *v = owl_variable_get_var(d, name, require_type);
-  if(v == NULL) return NULL;
+const void *owl_variable_get(const owl_variable *v)
+{
   return v->get_fn(v);
 }
 
-/* returns a reference */
-const char *owl_variable_get_string(const owl_vardict *d, const char *name) {
-  return owl_variable_get(d,name, OWL_VARIABLE_STRING);
+const char *owl_variable_get_string(const owl_variable *v)
+{
+  if (owl_variable_get_type(v) != OWL_VARIABLE_STRING) {
+    owl_function_error("Variable '%s' is not a string.", owl_variable_get_name(v));
+    return NULL;
+  }
+  return owl_variable_get(v);
 }
 
 /* returns a reference */
-const void *owl_variable_get_other(const owl_vardict *d, const char *name) {
-  return owl_variable_get(d,name, OWL_VARIABLE_OTHER);
-}
-
-int owl_variable_get_int(const owl_vardict *d, const char *name) {
-  const int *pi;
-  pi = owl_variable_get(d,name,OWL_VARIABLE_INT);
-  if (!pi) return(-1);
-  return(*pi);
-}
-
-int owl_variable_get_bool(const owl_vardict *d, const char *name) {
-  const int *pi;
-  pi = owl_variable_get(d,name,OWL_VARIABLE_BOOL);
-  if (!pi) return(-1);
-  return(*pi);
-}
-
-void owl_variable_describe(const owl_vardict *d, const char *name, owl_fmtext *fm) {
-  char defaultbuf[50];
-  char buf[1024];
-  int buflen = 1023;
-  owl_variable *v;
-
-  if (!name
-      || (v = owl_dict_find_element(d, name)) == NULL 
-      || !v->get_fn) {
-    snprintf(buf, buflen, "     No such variable '%s'\n", name);     
-    owl_fmtext_append_normal(fm, buf);
-    return;
+const void *owl_variable_get_other(const owl_variable *v)
+{
+  if (owl_variable_get_type(v) != OWL_VARIABLE_OTHER) {
+    owl_function_error("Variable '%s' is not type other.", owl_variable_get_name(v));
+    return NULL;
   }
-  if (v->type == OWL_VARIABLE_INT || v->type == OWL_VARIABLE_BOOL) {
-    v->get_tostring_fn(v, defaultbuf, 50, &(v->ival_default));
-  } else {
-    v->get_tostring_fn(v, defaultbuf, 50, v->pval_default);
-  }
-  snprintf(buf, buflen, OWL_TABSTR "%-20s - %s (default: '%s')\n", 
-		  v->name, 
-		  owl_variable_get_summary(v), defaultbuf);
-  owl_fmtext_append_normal(fm, buf);
+  return owl_variable_get(v);
 }
 
-void owl_variable_get_help(const owl_vardict *d, const char *name, owl_fmtext *fm) {
-  char buff[1024];
-  int bufflen = 1023;
-  owl_variable *v;
-
-  if (!name
-      || (v = owl_dict_find_element(d, name)) == NULL 
-      || !v->get_fn) {
-    owl_fmtext_append_normal(fm, "No such variable...\n");
-    return;
+int owl_variable_get_int(const owl_variable *v)
+{
+  if (owl_variable_get_type(v) != OWL_VARIABLE_INT) {
+    owl_function_error("Variable '%s' is an int.", owl_variable_get_name(v));
+    return -1;
   }
+  const int *pi = owl_variable_get(v);
+  if (!pi) return -1;
+  return *pi;
+}
+
+int owl_variable_get_bool(const owl_variable *v)
+{
+  if (owl_variable_get_type(v) != OWL_VARIABLE_BOOL) {
+    owl_function_error("Variable '%s' is a boolean.", owl_variable_get_name(v));
+    return -1;
+  }
+  const int *pi = owl_variable_get(v);
+  if (!pi) return -1;
+  return *pi;
+}
+
+void owl_variable_describe(const owl_variable *v, owl_fmtext *fm)
+{
+  char *tostring = owl_variable_get_default_tostring(v);
+  char *default_buf;
+
+  if (tostring)
+    default_buf = g_strdup_printf("'%s'", tostring);
+  else
+    default_buf = g_strdup("<null>");
+  owl_fmtext_appendf_normal(fm, OWL_TABSTR "%-20s - %s (default: %s)\n",
+                            owl_variable_get_name(v),
+                            owl_variable_get_summary(v), default_buf);
+  g_free(default_buf);
+  g_free(tostring);
+}
+
+void owl_variable_get_help(const owl_variable *v, owl_fmtext *fm) {
+  char *tostring;
 
   owl_fmtext_append_bold(fm, "OWL VARIABLE\n\n");
   owl_fmtext_append_normal(fm, OWL_TABSTR);
-  owl_fmtext_append_normal(fm, name);
+  owl_fmtext_append_normal(fm, owl_variable_get_name(v));
   owl_fmtext_append_normal(fm, " - ");
-  owl_fmtext_append_normal(fm, v->summary);
+  owl_fmtext_append_normal(fm, owl_variable_get_summary(v));
   owl_fmtext_append_normal(fm, "\n\n");
 
   owl_fmtext_append_normal(fm, "Current:        ");
-  owl_variable_get_tostring(d, name, buff, bufflen);
-  owl_fmtext_append_normal(fm, buff);
+  tostring = owl_variable_get_tostring(v);
+  owl_fmtext_append_normal(fm, (tostring ? tostring : "<null>"));
+  g_free(tostring);
   owl_fmtext_append_normal(fm, "\n\n");
 
 
-  if (v->type == OWL_VARIABLE_INT || v->type == OWL_VARIABLE_BOOL) {
-    v->get_tostring_fn(v, buff, bufflen, &(v->ival_default));
-  } else {
-    v->get_tostring_fn(v, buff, bufflen, v->pval_default);
-  }
+  tostring = owl_variable_get_default_tostring(v);
   owl_fmtext_append_normal(fm, "Default:        ");
-  owl_fmtext_append_normal(fm, buff);
+  owl_fmtext_append_normal(fm, (tostring ? tostring : "<null>"));
   owl_fmtext_append_normal(fm, "\n\n");
 
   owl_fmtext_append_normal(fm, "Valid Settings: ");
@@ -852,6 +939,7 @@ void owl_variable_get_help(const owl_vardict *d, const char *name, owl_fmtext *f
     owl_fmtext_append_normal(fm, owl_variable_get_description(v));
     owl_fmtext_append_normal(fm, "\n\n");
   }
+  g_free(tostring);
 }
 
 
@@ -869,7 +957,7 @@ const void *owl_variable_get_default(const owl_variable *v) {
 
 void owl_variable_delete_default(owl_variable *v)
 {
-  if (v->val) owl_free(v->val);
+  g_free(v->val);
 }
 
 /* default functions for booleans */
@@ -896,19 +984,16 @@ int owl_variable_bool_set_fromstring_default(owl_variable *v, const char *newval
   return (v->set_fn(v, &i));
 }
 
-int owl_variable_bool_get_tostring_default(const owl_variable *v, char* buf, int bufsize, const void *val) {
+CALLER_OWN char *owl_variable_bool_get_tostring_default(const owl_variable *v, const void *val)
+{
   if (val == NULL) {
-    snprintf(buf, bufsize, "<null>");
-    return -1;
+    return NULL;
   } else if (*(const int*)val == 0) {
-    snprintf(buf, bufsize, "off");
-    return 0;
+    return g_strdup("off");
   } else if (*(const int*)val == 1) {
-    snprintf(buf, bufsize, "on");
-    return 0;
+    return g_strdup("on");
   } else {
-    snprintf(buf, bufsize, "<invalid>");
-    return -1;
+    return g_strdup("<invalid>");
   }
 }
 
@@ -935,13 +1020,12 @@ int owl_variable_int_set_fromstring_default(owl_variable *v, const char *newval)
   return (v->set_fn(v, &i));
 }
 
-int owl_variable_int_get_tostring_default(const owl_variable *v, char* buf, int bufsize, const void *val) {
+CALLER_OWN char *owl_variable_int_get_tostring_default(const owl_variable *v, const void *val)
+{
   if (val == NULL) {
-    snprintf(buf, bufsize, "<null>");
-    return -1;
+    return NULL;
   } else {
-    snprintf(buf, bufsize, "%d", *(const int*)val);
-    return 0;
+    return g_strdup_printf("%d", *(const int*)val);
   } 
 }
 
@@ -951,9 +1035,9 @@ int owl_variable_enum_validate(const owl_variable *v, const void *newval) {
   char **enums;
   int nenums, val;
   if (newval == NULL) return(0);
-  enums = atokenize(v->validsettings, ",", &nenums);
-  if (enums == NULL) return(0);
-  atokenize_delete(enums, nenums);
+  enums = g_strsplit_set(v->validsettings, ",", 0);
+  nenums = g_strv_length(enums);
+  g_strfreev(enums);
   val = *(const int*)newval;
   if (val < 0 || val >= nenums) {
     return(0);
@@ -963,38 +1047,38 @@ int owl_variable_enum_validate(const owl_variable *v, const void *newval) {
 
 int owl_variable_enum_set_fromstring(owl_variable *v, const char *newval) {
   char **enums;
-  int nenums, i, val=-1;
+  int i, val=-1;
   if (newval == NULL) return(-1);
-  enums = atokenize(v->validsettings, ",", &nenums);
-  if (enums == NULL) return(-1);
-  for (i=0; i<nenums; i++) {
+  enums = g_strsplit_set(v->validsettings, ",", 0);
+  for (i = 0; enums[i] != NULL; i++) {
     if (0==strcmp(newval, enums[i])) {
       val = i;
     }
   }
-  atokenize_delete(enums, nenums);
+  g_strfreev(enums);
   if (val == -1) return(-1);
   return (v->set_fn(v, &val));
 }
 
-int owl_variable_enum_get_tostring(const owl_variable *v, char* buf, int bufsize, const void *val) {
+CALLER_OWN char *owl_variable_enum_get_tostring(const owl_variable *v, const void *val)
+{
   char **enums;
   int nenums, i;
+  char *tostring;
 
   if (val == NULL) {
-    snprintf(buf, bufsize, "<null>");
-    return -1;
+    return NULL;
   }
-  enums = atokenize(v->validsettings, ",", &nenums);
+  enums = g_strsplit_set(v->validsettings, ",", 0);
+  nenums = g_strv_length(enums);
   i = *(const int*)val;
   if (i<0 || i>=nenums) {
-    snprintf(buf, bufsize, "<invalid:%d>",i);
-    atokenize_delete(enums, nenums);
-    return(-1);
+    g_strfreev(enums);
+    return g_strdup_printf("<invalid:%d>", i);
   }
-  snprintf(buf, bufsize, "%s", enums[i]);
-  atokenize_delete(enums, nenums);
-  return 0;
+  tostring = g_strdup(enums[i]);
+  g_strfreev(enums);
+  return tostring;
 }
 
 /* default functions for stringeans */
@@ -1008,8 +1092,8 @@ int owl_variable_string_set_default(owl_variable *v, const void *newval) {
   if (v->validate_fn) {
     if (!v->validate_fn(v, newval)) return(-1);
   }
-  if (v->val) owl_free(v->val);
-  v->val = owl_strdup(newval);
+  g_free(v->val);
+  v->val = g_strdup(newval);
   return(0);
 }
 
@@ -1017,13 +1101,8 @@ int owl_variable_string_set_fromstring_default(owl_variable *v, const char *newv
   return (v->set_fn(v, newval));
 }
 
-int owl_variable_string_get_tostring_default(const owl_variable *v, char* buf, int bufsize, const void *val) {
-  if (val == NULL) {
-    snprintf(buf, bufsize, "<null>");
-    return -1;
-  } else {
-    snprintf(buf, bufsize, "%s", (const char*)val);
-    return 0;
-  }
+CALLER_OWN char *owl_variable_string_get_tostring_default(const owl_variable *v, const void *val)
+{
+  return g_strdup((const char*)val);
 }
 
