@@ -4,9 +4,9 @@
 #include <unistd.h>
 #include "owl.h"
 
-owl_zwrite *owl_zwrite_new(const char *line)
+CALLER_OWN owl_zwrite *owl_zwrite_new(const char *line)
 {
-  owl_zwrite *z = owl_malloc(sizeof *z);
+  owl_zwrite *z = g_new(owl_zwrite, 1);
   if (owl_zwrite_create_from_line(z, line) < 0) {
     owl_zwrite_delete(z);
     return NULL;
@@ -14,9 +14,9 @@ owl_zwrite *owl_zwrite_new(const char *line)
   return z;
 }
 
-int owl_zwrite_create_from_line(owl_zwrite *z, const char *line)
+G_GNUC_WARN_UNUSED_RESULT int owl_zwrite_create_from_line(owl_zwrite *z, const char *line)
 {
-  int argc, badargs, myargc, i, len;
+  int argc, badargs, myargc;
   char **argv;
   const char *const *myargv;
   char *msg = NULL;
@@ -33,8 +33,8 @@ int owl_zwrite_create_from_line(owl_zwrite *z, const char *line)
   z->message=NULL;
   z->cc=0;
   z->noping=0;
-  owl_list_create(&(z->recips));
-  z->zwriteline = owl_strdup(line);
+  z->recips = g_ptr_array_new();
+  z->zwriteline = g_strdup(line);
 
   /* parse the command line for options */
   argv=owl_parseline(line, &argc);
@@ -45,7 +45,7 @@ int owl_zwrite_create_from_line(owl_zwrite *z, const char *line)
   }
   myargc=argc;
   if (myargc && *(myargv[0])!='-') {
-    z->cmd=owl_strdup(myargv[0]);
+    z->cmd=g_strdup(myargv[0]);
     myargc--;
     myargv++;
   }
@@ -96,7 +96,7 @@ int owl_zwrite_create_from_line(owl_zwrite *z, const char *line)
 	break;
       }
       /* we must already have users or a class or an instance */
-      if (owl_list_get_size(&(z->recips))<1 && (!z->class) && (!z->inst)) {
+      if (z->recips->len < 1 && (!z->class) && (!z->inst)) {
 	badargs=1;
 	break;
       }
@@ -104,20 +104,7 @@ int owl_zwrite_create_from_line(owl_zwrite *z, const char *line)
       /* Once we have -m, gobble up everything else on the line */
       myargv++;
       myargc--;
-      len = 0;
-      for (i=0;i<myargc;i++) {
-        len += strlen(myargv[i]) + 1;
-      }
-      len++;    /* NULL after the last trailing ' ' in the loop below. */
-      msg = owl_malloc(len);
-      msg[0] = '\0';
-      while (myargc) {
-        strcat(msg, myargv[0]);
-        strcat(msg, " ");
-        myargc--;
-        myargv++;
-      }
-      msg[strlen(msg)-1] = '\0';
+      msg = g_strjoinv(" ", (char**)myargv);
       break;
     } else if (!strcmp(myargv[0], "-C")) {
       z->cc=1;
@@ -129,13 +116,13 @@ int owl_zwrite_create_from_line(owl_zwrite *z, const char *line)
       myargc--;
     } else {
       /* anything unattached is a recipient */
-      owl_list_append_element(&(z->recips), owl_validate_utf8(myargv[0]));
+      g_ptr_array_add(z->recips, owl_validate_utf8(myargv[0]));
       myargv++;
       myargc--;
     }
   }
 
-  owl_parse_delete(argv, argc);
+  g_strfreev(argv);
 
   if (badargs) {
     return(-1);
@@ -143,21 +130,21 @@ int owl_zwrite_create_from_line(owl_zwrite *z, const char *line)
 
   if (z->class == NULL &&
       z->inst == NULL &&
-      owl_list_get_size(&(z->recips))==0) {
+      z->recips->len == 0) {
     owl_function_error("You must specify a recipient for zwrite");
     return(-1);
   }
 
   /* now deal with defaults */
-  if (z->class==NULL) z->class=owl_strdup("message");
-  if (z->inst==NULL) z->inst=owl_strdup("personal");
-  if (z->realm==NULL) z->realm=owl_strdup("");
-  if (z->opcode==NULL) z->opcode=owl_strdup("");
+  if (z->class==NULL) z->class=g_strdup("message");
+  if (z->inst==NULL) z->inst=g_strdup("personal");
+  if (z->realm==NULL) z->realm=g_strdup("");
+  if (z->opcode==NULL) z->opcode=g_strdup("");
   /* z->message is allowed to stay NULL */
 
   if(msg) {
     owl_zwrite_set_message(z, msg);
-    owl_free(msg);
+    g_free(msg);
   }
 
   return(0);
@@ -174,7 +161,7 @@ void owl_zwrite_populate_zsig(owl_zwrite *z)
 
 void owl_zwrite_send_ping(const owl_zwrite *z)
 {
-  int i, j;
+  int i;
   char *to;
 
   if (z->noping) return;
@@ -183,17 +170,11 @@ void owl_zwrite_send_ping(const owl_zwrite *z)
     return;
   }
 
-  /* if there are no recipients we won't send a ping, which
-     is what we want */
-  j=owl_list_get_size(&(z->recips));
-  for (i=0; i<j; i++) {
-    if (strcmp(z->realm, "")) {
-      to = owl_sprintf("%s@%s", (const char *) owl_list_get_element(&(z->recips), i), z->realm);
-    } else {
-      to = owl_strdup(owl_list_get_element(&(z->recips), i));
-    }
-    send_ping(to, z->class, z->inst);
-    owl_free(to);
+  for (i = 0; i < z->recips->len; i++) {
+    to = owl_zwrite_get_recip_n_with_realm(z, i);
+    if (owl_zwrite_recip_is_personal(to))
+      send_ping(to, z->class, z->inst);
+    g_free(to);
   }
 
 }
@@ -201,41 +182,36 @@ void owl_zwrite_send_ping(const owl_zwrite *z)
 /* Set the message with no post-processing*/
 void owl_zwrite_set_message_raw(owl_zwrite *z, const char *msg)
 {
-  if (z->message) owl_free(z->message);
+  g_free(z->message);
   z->message = owl_validate_utf8(msg);
 }
 
 void owl_zwrite_set_message(owl_zwrite *z, const char *msg)
 {
-  int i, j;
-  char *toline = NULL;
+  int i;
+  GString *message;
   char *tmp = NULL, *tmp2;
 
-  if (z->message) owl_free(z->message);
+  g_free(z->message);
 
-  j=owl_list_get_size(&(z->recips));
-  if (j>0 && z->cc) {
-    toline = owl_strdup( "CC: ");
-    for (i=0; i<j; i++) {
-      tmp = toline;
-      if (strcmp(z->realm, "")) {
-        toline = owl_sprintf( "%s%s@%s ", toline, (const char *) owl_list_get_element(&(z->recips), i), z->realm);
-      } else {
-        toline = owl_sprintf( "%s%s ", toline, (const char *) owl_list_get_element(&(z->recips), i));
-      }
-      owl_free(tmp);
+  if (z->cc && owl_zwrite_is_personal(z)) {
+    message = g_string_new("CC: ");
+    for (i = 0; i < z->recips->len; i++) {
+      tmp = owl_zwrite_get_recip_n_with_realm(z, i);
+      g_string_append_printf(message, "%s ", tmp);
+      g_free(tmp);
       tmp = NULL;
     }
     tmp = owl_validate_utf8(msg);
     tmp2 = owl_text_expand_tabs(tmp);
-    z->message=owl_sprintf("%s\n%s", toline, tmp2);
-    owl_free(toline);
-    owl_free(tmp);
-    owl_free(tmp2);
+    g_string_append_printf(message, "\n%s", tmp2);
+    z->message = g_string_free(message, false);
+    g_free(tmp);
+    g_free(tmp2);
   } else {
     tmp=owl_validate_utf8(msg);
     z->message=owl_text_expand_tabs(tmp);
-    owl_free(tmp);
+    g_free(tmp);
   }
 }
 
@@ -253,31 +229,26 @@ int owl_zwrite_is_message_set(const owl_zwrite *z)
 
 int owl_zwrite_send_message(const owl_zwrite *z)
 {
-  int i, j, ret = 0;
+  int i, ret = 0;
   char *to = NULL;
 
   if (z->message==NULL) return(-1);
 
-  j=owl_list_get_size(&(z->recips));
-  if (j>0) {
-    for (i=0; i<j; i++) {
-      if (strcmp(z->realm, "")) {
-        to = owl_sprintf("%s@%s", (const char *) owl_list_get_element(&(z->recips), i), z->realm);
-      } else {
-        to = owl_strdup( owl_list_get_element(&(z->recips), i));
-      }
+  if (z->recips->len > 0) {
+    for (i = 0; i < z->recips->len; i++) {
+      to = owl_zwrite_get_recip_n_with_realm(z, i);
       ret = send_zephyr(z->opcode, z->zsig, z->class, z->inst, to, z->message);
       /* Abort on the first error, to match the zwrite binary. */
       if (ret != 0)
         break;
-      owl_free(to);
+      g_free(to);
       to = NULL;
     }
   } else {
-    to = owl_sprintf( "@%s", z->realm);
+    to = g_strdup_printf( "@%s", z->realm);
     ret = send_zephyr(z->opcode, z->zsig, z->class, z->inst, to, z->message);
   }
-  owl_free(to);
+  g_free(to);
   return ret;
 }
 
@@ -313,7 +284,7 @@ const char *owl_zwrite_get_opcode(const owl_zwrite *z)
 
 void owl_zwrite_set_opcode(owl_zwrite *z, const char *opcode)
 {
-  if (z->opcode) owl_free(z->opcode);
+  g_free(z->opcode);
   z->opcode=owl_validate_utf8(opcode);
 }
 
@@ -330,62 +301,113 @@ const char *owl_zwrite_get_zsig(const owl_zwrite *z)
 
 void owl_zwrite_set_zsig(owl_zwrite *z, const char *zsig)
 {
-  if(z->zsig) owl_free(z->zsig);
-  z->zsig = owl_strdup(zsig);
-}
-
-void owl_zwrite_get_recipstr(const owl_zwrite *z, char *buff)
-{
-  int i, j;
-
-  strcpy(buff, "");
-  j=owl_list_get_size(&(z->recips));
-  for (i=0; i<j; i++) {
-    strcat(buff, owl_list_get_element(&(z->recips), i));
-    strcat(buff, " ");
-  }
-  buff[strlen(buff)-1]='\0';
+  g_free(z->zsig);
+  z->zsig = g_strdup(zsig);
 }
 
 int owl_zwrite_get_numrecips(const owl_zwrite *z)
 {
-  return(owl_list_get_size(&(z->recips)));
+  return z->recips->len;
 }
 
 const char *owl_zwrite_get_recip_n(const owl_zwrite *z, int n)
 {
-  return(owl_list_get_element(&(z->recips), n));
+  return z->recips->pdata[n];
 }
 
-int owl_zwrite_is_personal(const owl_zwrite *z)
+/* Caller must free the result. */
+CALLER_OWN char *owl_zwrite_get_recip_n_with_realm(const owl_zwrite *z, int n)
+{
+  if (z->realm[0]) {
+    return g_strdup_printf("%s@%s", owl_zwrite_get_recip_n(z, n), z->realm);
+  } else {
+    return g_strdup(owl_zwrite_get_recip_n(z, n));
+  }
+}
+
+bool owl_zwrite_recip_is_personal(const char *recipient)
+{
+  return recipient[0] && recipient[0] != '@';
+}
+
+bool owl_zwrite_is_personal(const owl_zwrite *z)
 {
   /* return true if at least one of the recipients is personal */
-  int i, j;
-  char *foo;
+  int i;
 
-  j=owl_list_get_size(&(z->recips));
-  for (i=0; i<j; i++) {
-    foo=owl_list_get_element(&(z->recips), i);
-    if (foo[0]!='@') return(1);
-  }
-  return(0);
+  for (i = 0; i < z->recips->len; i++)
+    if (owl_zwrite_recip_is_personal(z->recips->pdata[i]))
+      return true;
+  return false;
 }
 
 void owl_zwrite_delete(owl_zwrite *z)
 {
   owl_zwrite_cleanup(z);
-  owl_free(z);
+  g_free(z);
 }
 
 void owl_zwrite_cleanup(owl_zwrite *z)
 {
-  owl_list_cleanup(&(z->recips), &owl_free);
-  if (z->cmd) owl_free(z->cmd);
-  if (z->zwriteline) owl_free(z->zwriteline);
-  if (z->class) owl_free(z->class);
-  if (z->inst) owl_free(z->inst);
-  if (z->opcode) owl_free(z->opcode);
-  if (z->realm) owl_free(z->realm);
-  if (z->message) owl_free(z->message);
-  if (z->zsig) owl_free(z->zsig);
+  owl_ptr_array_free(z->recips, g_free);
+  g_free(z->cmd);
+  g_free(z->zwriteline);
+  g_free(z->class);
+  g_free(z->inst);
+  g_free(z->opcode);
+  g_free(z->realm);
+  g_free(z->message);
+  g_free(z->zsig);
+}
+
+/*
+ * Returns a zwrite line suitable for replying, specifically the
+ * message field is stripped out. Result should be freed with
+ * g_free.
+ *
+ * If not a CC, only the recip_index'th user will be replied to.
+ */
+CALLER_OWN char *owl_zwrite_get_replyline(const owl_zwrite *z, int recip_index)
+{
+  /* Match ordering in zwrite help. */
+  GString *buf = g_string_new("");
+  int i;
+
+  /* Disturbingly, it is apparently possible to z->cmd to be null if
+   * owl_zwrite_create_from_line got something starting with -. And we
+   * can't kill it because this is exported to perl. */
+  owl_string_append_quoted_arg(buf, z->cmd ? z->cmd : "zwrite");
+  if (z->noping) {
+    g_string_append(buf, " -n");
+  }
+  if (z->cc) {
+    g_string_append(buf, " -C");
+  }
+  if (strcmp(z->class, "message")) {
+    g_string_append(buf, " -c ");
+    owl_string_append_quoted_arg(buf, z->class);
+  }
+  if (strcmp(z->inst, "personal")) {
+    g_string_append(buf, " -i ");
+    owl_string_append_quoted_arg(buf, z->inst);
+  }
+  if (z->realm && z->realm[0] != '\0') {
+    g_string_append(buf, " -r ");
+    owl_string_append_quoted_arg(buf, z->realm);
+  }
+  if (z->opcode && z->opcode[0] != '\0') {
+    g_string_append(buf, " -O ");
+    owl_string_append_quoted_arg(buf, z->opcode);
+  }
+  if (z->cc) {
+    for (i = 0; i < z->recips->len; i++) {
+      g_string_append_c(buf, ' ');
+      owl_string_append_quoted_arg(buf, z->recips->pdata[i]);
+    }
+  } else if (recip_index < z->recips->len) {
+    g_string_append_c(buf, ' ');
+    owl_string_append_quoted_arg(buf, z->recips->pdata[recip_index]);
+  }
+
+  return g_string_free(buf, false);
 }
